#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

#Module containing vtk visualizing classes:




#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#Misc
import sys
import time
import os, os.path

#PyFRAP modules
from pyfrp.modules.pyfrp_term_module import *

#PyQT
from PyQt4 import QtGui, QtCore

# VTK
import vtk
from vtk.qt4.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor



class vtkSimVisualizer(QtGui.QMainWindow):
	
	"""Basic simulation visualizing class.
	
	.. note:: Only works if the simulation results 
	   are saved in embryo via ``embryo.simulation.saveSim=True``.
	
	.. image:: ../imgs/pyfrp_gui_vtk/vtkSimVisualizer.png
		
	Args:
		embryo (pyfrp.subclasses.pyfrp_embryo.embryo): An embryo
	
	"""
	
	def __init__(self, embryo,  parent=None):
		
		super(vtkSimVisualizer, self).__init__(parent)
		
		# Load embryo
		self.embryo=embryo
		
		# Check if simulation is saved
		if len(self.embryo.simulation.vals)==0:
			printWarning("Embryo does not have saved simulation. Rerun simulation with saveSim=True.")
			return
			
		# To numpy array 
		self.embryo.simulation.vals=np.array(self.embryo.simulation.vals)
		
		# Frame
		self.frame = QtGui.QFrame()
		
		# Vtk widgets
		self.vtkWidget,self.ren,self.iren=self.initVTKWidget(self.frame)
		
		# Add slider for 3D
		self.slider,self.label=self.initSlider(0, len(self.embryo.simulation.tvecSim)-1,self.sliderCallback)
		
		# Show simulation visualization
		self.initSimVis()
		
		# Add Layout
		self.layout=QtGui.QGridLayout()
		self.layout.addWidget(self.vtkWidget,1,1)
		self.layout.addWidget(self.slider,2,1)
		self.layout.addWidget(self.label,3,1)
		
		# Set central widget
		self.frame.setLayout(self.layout)
		self.setCentralWidget(self.frame)
		
		# Window title
		self.setWindowTitle('vtkSimVisualizer of embryo ' + self.embryo.name)
		
		# Show everything
		self.show()
		self.iren.Initialize()

	def initSlider(self,sMin,sMax,callback):
		
		"""Sets up slider and corresponding label with given limits and callback.
		
		Args:
			sMin (int): Minimum index of slider.
			sMax (int): Maximum index of slider.
			callback (function): Callback function of slider.
			
		Returns:
			tuple: Tuple containing:
			
				* slider (QtGui.QSlider)
				* lbl (QtGui.QLabel)
		"""
		
		slider = QtGui.QSlider()
		slider.setRange(sMin,sMax)
		slider.valueChanged.connect(callback)
		slider.setOrientation(QtCore.Qt.Horizontal)
		lbl=QtGui.QLabel("")
		return slider,lbl
		
	def initVTKWidget(self,frame):
		
		"""Sets up vtkWidget inside frame.
		
		Also sets up corresponding renderer and interactor.
		
		Args:
			frame (QtGui.QFrame): Parenting frame.
			
		Returns:
			tuple: Tuple containing:
			
				* vtkWidget (vtk.qt4.QVTKRenderWindowInteractor.QVTKRenderWindowInteractor): Qt vtk window interactor.
				* ren (vtk.vtkRenderer)
				* iren (vtk.vtkRenderWindowInteractor)
				
		"""
		
		vtkWidget = QVTKRenderWindowInteractor(frame)
		ren = vtk.vtkRenderer()
		vtkWidget.GetRenderWindow().AddRenderer(ren)
		iren = vtkWidget.GetRenderWindow().GetInteractor()
		iren.SetInteractorStyle(vtk.vtkInteractorStyleTrackballCamera())
		ren.ResetCamera()
		
		return vtkWidget, ren, iren
	
	def showVals(self,idx):
		
		"""Updates simulation values in renderer.
		
		Args:
			idx (int): Index of timepoint.
		
		"""
		
		# Convert to right cmap
		vals=self.embryo.simulation.vals[idx]/self.embryo.simulation.vals.max()
		vals=255*self.cm(vals)
		vals=vals[:,:3]

		self.initScalarArray()
		for i,val in enumerate(vals):
			self.scalar.InsertNextTupleValue(val)	
		self.grid.GetCellData().SetScalars(self.scalar)
		self.grid.Modified()
		self.iren.Render()
		
		try:
			self.CutMesh()
		except AttributeError:
			pass
	
	def initSimVis(self):
		
		"""Inits the simulation renderer.
		
		This contains the following steps:
		
			* Initializing points.
			* Initializing grid ontop of points.
			* Initialzing tetrahedras.
			* Initializing a scalar array.
			* Initializing a mapper and actor.
			* Initializing a color table.
			* Shows initial time point in renderer.
			
		"""
		
		self.initPoints()
		self.initGrid()
		if self.embryo.geometry.dim==2:
			self.initTriangles()
		else:
			self.initTetras()
		self.initScalarArray()
		self.initMapper()
		self.initActor()
		self.initColorTable()
		self.showVals(0)
	
	def initActor(self):
		
		"""Sets up simulation actor."""
		
		self.actor = vtk.vtkActor()
		self.actor.SetMapper(self.mapper)
		self.ren.AddActor(self.actor)
		
	def initMapper(self):
		
		"""Sets up simulation mapper."""
		
		self.mapper = vtk.vtkDataSetMapper()
		if vtk.VTK_MAJOR_VERSION <= 5:
			self.mapper.SetInputConnection(self.grid.GetProducerPort())
		else:
			self.mapper.SetInputData(self.grid)	
	
	def initGrid(self):
		
		"""Sets up simulation grid."""
		
		self.grid = vtk.vtkUnstructuredGrid()
		self.grid.SetPoints(self.points)
		
		return self.grid
		
	def initTetras(self):
		
		"""Sets up tetrahedras describing mesh."""
		
		verts=self.embryo.simulation.mesh.mesh._getOrderedCellVertexIDs().T
		
		cellArray = vtk.vtkCellArray()
		for j,vert in enumerate(verts):

			tetra = vtk.vtkTetra()
			
			for i,v in enumerate(vert):
				tetra.GetPointIds().SetId(i, v)
			cellArray.InsertNextCell(tetra)
			
		self.grid.SetCells(vtk.VTK_TETRA, cellArray)
	
	def initTriangles(self):
		
		"""Sets up triangles describing mesh."""
		
		verts=self.embryo.simulation.mesh.mesh._getOrderedCellVertexIDs().T
		
		cellArray = vtk.vtkCellArray()
		for j,vert in enumerate(verts):

			tetra = vtk.vtkTriangle()
			
			for i,v in enumerate(vert):
				tetra.GetPointIds().SetId(i, v)
			cellArray.InsertNextCell(tetra)
			
		self.grid.SetCells(vtk.VTK_TRIANGLE, cellArray)
	
	def initScalarArray(self):	
		
		"""Sets up scalar array describing tetrahedra values."""
		
		self.scalar = vtk.vtkUnsignedCharArray()
		self.scalar.SetNumberOfComponents(3)
		self.scalar.SetName("Colors")
		
		return self.scalar
		
	def initPoints(self):
		
		"""Sets up point array."""
		
		if self.embryo.geometry.dim==2:
			x,y=self.embryo.simulation.mesh.mesh.vertexCoords
		else:	
			x,y,z=self.embryo.simulation.mesh.mesh.vertexCoords
		
		self.points = vtk.vtkPoints()
		for i in range(len(x)):
			if self.embryo.geometry.dim==2:
				self.points.InsertNextPoint(x[i], y[i],0)
			else:	
				self.points.InsertNextPoint(x[i], y[i],z[i])
		
		return self.points
	
	def getRenderer(self):
		
		"""Returns renderer."""
		
		return self.ren
	
	def sliderCallback(self):
		
		"""Call back function for slider movement."""
		
		index = self.sender().value()
		self.label.setText("t = "+str(self.embryo.simulation.tvecSim[index]))
		self.showVals(index)
		return
	
	def initColorTable(self):
		
		"""Sets up color table."""
		
		from matplotlib import cm
		
		self.cm=cm.jet


class vtkSimVisualizerCutter(QtGui.QMainWindow):
	
	"""Simulation visualizing class that allows to draw the simulation in 3D and slice a plane 
	out of the 3D simulation.
	
	.. note:: Only works for 3D embryos that already have been simulated and the simulation results 
	   are saved in embryo via ``embryo.simulation.saveSim=True``.
	
	.. image:: ../imgs/pyfrp_gui_vtk/vtkSimVisualizerCutter.png
		
	Args:
		embryo (pyfrp.subclasses.pyfrp_embryo.embryo): An embryo
	
	"""
	
	def __init__(self, embryo,  parent=None):
		
		super(vtkSimVisualizerCutter, self).__init__(parent)
		
		# Load embryo
		self.embryo=embryo
		
		# Check if simulation is saved
		if len(self.embryo.simulation.vals)==0:
			printWarning("Embryo does not have saved simulation. Rerun simulation with saveSim=True.")
			
		# Check if 3D embryo
		if self.embryo.geometry.dim==2:
			printError("vtkSimVisualizerCutter does only work and make sense for 3D geometries. Use vtkSimVisualizer instead.")
		
		# To numpy array 
		self.embryo.simulation.vals=np.array(self.embryo.simulation.vals)
		
		# Frame
		self.frame = QtGui.QFrame()
		
		# Vtk widgets
		self.vtkWidget,self.ren,self.iren=self.initVTKWidget(self.frame)
		self.vtkWidgetCut,self.renCut,self.irenCut=self.initVTKWidget(self.frame)
		
		# Add slider for 3D
		self.slider,self.label=self.initSlider(0, len(self.embryo.simulation.tvecSim)-1,self.sliderCallback)
		
		# Setup 3D simulation plot
		self.init3D()
		
		# Plane used for cutting
		self.plane = vtk.vtkPlane()
		
		# Cutter
		self.cutter = vtk.vtkCutter()
		
		# Plane widget
		self.initPlaneWidget()
		
		# Add Layout
		self.layout=QtGui.QGridLayout()
		self.layout.addWidget(self.vtkWidget,1,1)
		self.layout.addWidget(self.vtkWidgetCut,1,2)
		self.layout.addWidget(self.slider,2,1)
		self.layout.addWidget(self.label,3,1)
		
		# Set central widget
		self.frame.setLayout(self.layout)
		self.setCentralWidget(self.frame)
		
		self.setWindowTitle('vtkSimVisualizerCutter of embryo ' + self.embryo.name)
		
		# Run cut mesh once
		self.CutMesh()
		
		# Show everything
		self.show()
		self.iren.Initialize()
		self.irenCut.Initialize()
	
	def closeIt(self):
		self.close()
	
	def initSlider(self,sMin,sMax,callback):
		
		"""Sets up slider and corresponding label with given limits and callback.
		
		Args:
			sMin (int): Minimum index of slider.
			sMax (int): Maximum index of slider.
			callback (function): Callback function of slider.
			
		Returns:
			tuple: Tuple containing:
			
				* slider (QtGui.QSlider)
				* lbl (QtGui.QLabel)
		"""
		
		slider = QtGui.QSlider()
		slider.setRange(sMin,sMax)
		slider.valueChanged.connect(callback)
		slider.setOrientation(QtCore.Qt.Horizontal)
		lbl=QtGui.QLabel("")
		return slider,lbl
		
	def initVTKWidget(self,frame):
		
		"""Sets up vtkWidget inside frame.
		
		Also sets up corresponding renderer and interactor.
		
		Args:
			frame (QtGui.QFrame): Parenting frame.
			
		Returns:
			tuple: Tuple containing:
			
				* vtkWidget (vtk.qt4.QVTKRenderWindowInteractor.QVTKRenderWindowInteractor): Qt vtk window interactor.
				* ren (vtk.vtkRenderer)
				* iren (vtk.vtkRenderWindowInteractor)
				
		"""
		
		vtkWidget = QVTKRenderWindowInteractor(frame)
		ren = vtk.vtkRenderer()
		vtkWidget.GetRenderWindow().AddRenderer(ren)
		iren = vtkWidget.GetRenderWindow().GetInteractor()
		iren.SetInteractorStyle(vtk.vtkInteractorStyleTrackballCamera())
		ren.ResetCamera()
		
		return vtkWidget, ren, iren
	
	def initPlaneWidget(self):
		
		"""Sets up vtkImplicitPlaneWidget for plane selection.
		
		Sets also up necessary actor and mapper.
		
		Returns:
			vtk.vtkImplicitPlaneWidget: Widget.
		
		"""
		
		# Initialize a plane widget
		self.planeWidget = vtk.vtkImplicitPlaneWidget()
		self.planeWidget.SetInteractor(self.iren)
		self.planeWidget.SetPlaceFactor(1.25)
		self.planeWidget.SetInput(self.grid)
		self.planeWidget.PlaceWidget()
		
		# Callback connection of interaction event
		self.planeWidget.AddObserver("InteractionEvent", self.StartPlaneCallback)
		self.planeWidget.AddObserver("EndInteractionEvent",self.EndPlaneCallback)
		
		# Mapper
		self.selectMapper = vtk.vtkDataSetMapper()
		self.selectMapper.SetInput(self.grid)
		
		# Actor
		self.selectActor = vtk.vtkLODActor()
		self.selectActor.SetMapper(self.selectMapper)
		self.selectActor.GetProperty().SetColor(0, 1, 0)
		self.selectActor.VisibilityOff()
		self.selectActor.SetScale(1.01, 1.01, 1.01)
		
		self.ren.AddActor(self.selectActor)
		
		# Place widget at right position
		self.planeWidget.SetNormal(0,0,1)
		self.planeWidget.SetOrigin(self.grid.GetCenter())
		
		
		return self.planeWidget
		
	def StartPlaneCallback(self,obj, event):
		
		"""Callback for interaction event.
		"""
		
		obj.GetPlane(self.plane)
		self.selectActor.VisibilityOn()
	
	def EndPlaneCallback(self,widget,event_string):
		
		"""Callback for end of interaction event.
		"""
		
		self.CutMesh()
	
	def CutMesh(self):
		
		"""Cuts plane out of 3D simulation object.
		
		Grabs plane and creates and actor and mapper to
		display cut out plane in second renderer.
		
		"""
		
		self.cutter.SetInput(self.grid)
		self.planeWidget.GetPlane(self.plane)
		self.cutter.SetCutFunction(self.plane)
		self.cutter.Update()
		
		self.Actor = vtk.vtkActor()
		
		mapper = vtk.vtkPolyDataMapper()
		
		mapper.SetInputConnection(self.cutter.GetOutputPort())
		self.Actor.SetMapper(mapper)
		self.renCut.AddActor(self.Actor)
		
		self.cutter.GetOutput().Modified()
		self.irenCut.Render()
		
	def showVals(self,idx):
		
		"""Updates simulation values in renderer.
		
		Args:
			idx (int): Index of timepoint.
		
		"""
		
		# Convert to right cmap
		vals=self.embryo.simulation.vals[idx]/self.embryo.simulation.vals.max()
		vals=255*self.cm(vals)
		vals=vals[:,:3]

		self.initScalarArray()
		for i,val in enumerate(vals):
			self.scalar.InsertNextTupleValue(val)	
		self.grid.GetCellData().SetScalars(self.scalar)
		self.grid.Modified()
		self.iren.Render()
		
		try:
			self.CutMesh()
		except AttributeError:
			pass
		
	def init3D(self):
		
		"""Inits the simulation renderer.
		
		This contains the following steps:
		
			* Initializing points.
			* Initializing grid ontop of points.
			* Initialzing tetrahedras.
			* Initializing a scalar array.
			* Initializing a mapper and actor.
			* Initializing a color table.
			* Shows initial time point in renderer.
			
		"""
		
		self.initPoints()
		self.initGrid()
		self.initTetras()
		self.initScalarArray()
		self.initMapper()
		self.initActor()
		self.initColorTable()
		self.showVals(0)
	
	def initActor(self):
		
		"""Sets up simulation actor."""
		
		self.actor = vtk.vtkActor()
		self.actor.SetMapper(self.mapper)
		self.ren.AddActor(self.actor)
		
	def initMapper(self):
		
		"""Sets up simulation mapper."""
		
		self.mapper = vtk.vtkDataSetMapper()
		if vtk.VTK_MAJOR_VERSION <= 5:
			self.mapper.SetInputConnection(self.grid.GetProducerPort())
		else:
			self.mapper.SetInputData(self.grid)	
	
	def initGrid(self):
		
		"""Sets up simulation grid."""
		
		self.grid = vtk.vtkUnstructuredGrid()
		self.grid.SetPoints(self.points)
		
		return self.grid
		
	def initTetras(self):
		
		"""Sets up tetrahedras describing mesh."""
		
		verts=self.embryo.simulation.mesh.mesh._getOrderedCellVertexIDs().T
		
		cellArray = vtk.vtkCellArray()
		for j,vert in enumerate(verts):

			tetra = vtk.vtkTetra()
			
			for i,v in enumerate(vert):
				tetra.GetPointIds().SetId(i, v)
			cellArray.InsertNextCell(tetra)
			
		self.grid.SetCells(vtk.VTK_TETRA, cellArray)
		
	def initScalarArray(self):	
		
		"""Sets up scalar array describing tetrahedra values."""
		
		self.scalar = vtk.vtkUnsignedCharArray()
		self.scalar.SetNumberOfComponents(3)
		self.scalar.SetName("Colors")
		
		return self.scalar
		
	def initPoints(self):
		
		"""Sets up point array."""
		
		x,y,z=self.embryo.simulation.mesh.mesh.vertexCoords
		
		self.points = vtk.vtkPoints()
		for i in range(len(x)):
			self.points.InsertNextPoint(x[i], y[i],z[i])
	
		return self.points
	
	def getRenderer(self):
		
		"""Returns renderer."""
		
		return self.ren
	
	def sliderCallback(self):
		
		"""Call back function for slider movement."""
		
		index = self.sender().value()
		self.label.setText("t = "+str(self.embryo.simulation.tvecSim[index]))
		self.showVals(index)
		return
	
	def initColorTable(self):
		
		"""Sets up color table."""
		
		from matplotlib import cm
		
		self.cm=cm.jet


class vtkSimVisualizerCutterBySub(vtkSimVisualizer):
	
	"""Simulation visualizing class that allows to draw the simulation in 3D and slice a plane 
	out of the 3D simulation.
	
	This one subclasses from vtkSimVisualizer. 
	
	.. warning:: Currently not working. Crashes at ``self.irenCut.Initialize()`` with ``segFault``. Currently no way to fix this, 
	   hence vtkSimVisualizerCutter is own instance instead of subclassing.
	
	Args:
		embryo (pyfrp.subclasses.pyfrp_embryo.embryo): An embryo
	
	"""
	
	def __init__(self, embryo,  parent=None):
		
		#super(vtkSimVisualizerCutter, self).__init__(embryo,parent)
		vtkSimVisualizer.__init__(self,embryo,parent=parent)
		
		# Check if 3D embryo
		if self.embryo.geometry.dim==2:
			printWarning("vtkSimVisualizerCutter does only work and make sense for 3D geometries. Will not initialize cutter and fall back tostandad vtkSimVisualizer.")
			return
		
		self.vtkWidgetCut,self.renCut,self.irenCut=self.initVTKWidget(self.frame)
		
		# Plane used for cutting
		self.plane = vtk.vtkPlane()
		
		# Cutter
		self.cutter = vtk.vtkCutter()
		
		# Plane widget
		self.initPlaneWidget()
		
		# Add Layout
		self.layout.addWidget(self.vtkWidgetCut,1,2)
	
		# Run cut mesh once
		self.CutMesh()
	
		self.irenCut.Initialize()
		
	def initPlaneWidget(self):
		
		"""Sets up vtkImplicitPlaneWidget for plane selection.
		
		Sets also up necessary actor and mapper.
		
		Returns:
			vtk.vtkImplicitPlaneWidget: Widget.
		
		"""
		
		# Initialize a plane widget
		self.planeWidget = vtk.vtkImplicitPlaneWidget()
		self.planeWidget.SetInteractor(self.iren)
		self.planeWidget.SetPlaceFactor(1.25)
		self.planeWidget.SetInput(self.grid)
		self.planeWidget.PlaceWidget()
		
		# Callback connection of interaction event
		self.planeWidget.AddObserver("InteractionEvent", self.StartPlaneCallback)
		self.planeWidget.AddObserver("EndInteractionEvent",self.EndPlaneCallback)
		
		# Mapper
		self.selectMapper = vtk.vtkDataSetMapper()
		self.selectMapper.SetInput(self.grid)
		
		# Actor
		self.selectActor = vtk.vtkLODActor()
		self.selectActor.SetMapper(self.selectMapper)
		self.selectActor.GetProperty().SetColor(0, 1, 0)
		self.selectActor.VisibilityOff()
		self.selectActor.SetScale(1.01, 1.01, 1.01)
		
		self.ren.AddActor(self.selectActor)
		
		# Place widget at right position
		self.planeWidget.SetNormal(0,0,1)
		self.planeWidget.SetOrigin(self.grid.GetCenter())
		
		
		return self.planeWidget
		
	def StartPlaneCallback(self,obj, event):
		
		"""Callback for interaction event.
		"""
		
		obj.GetPlane(self.plane)
		self.selectActor.VisibilityOn()
	
	def EndPlaneCallback(self,widget,event_string):
		
		"""Callback for end of interaction event.
		"""
		
		self.CutMesh()
	
	def CutMesh(self):
		
		"""Cuts plane out of 3D simulation object.
		
		Grabs plane and creates and actor and mapper to
		display cut out plane in second renderer.
		
		"""
		
		self.cutter.SetInput(self.grid)
		self.planeWidget.GetPlane(self.plane)
		self.cutter.SetCutFunction(self.plane)
		self.cutter.Update()
		
		self.Actor = vtk.vtkActor()
		
		mapper = vtk.vtkPolyDataMapper()
		
		mapper.SetInputConnection(self.cutter.GetOutputPort())
		self.Actor.SetMapper(mapper)
		self.renCut.AddActor(self.Actor)
		
		self.cutter.GetOutput().Modified()
		self.irenCut.Render()
	




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PyFRAP: A Python based FRAP analysis tool box. Subclass module.
"""

#PyFRAP classes
from . import pyfrp_analysis
from . import pyfrp_conf
from . import pyfrp_fit
from . import pyfrp_mesh
from . import pyfrp_simulation
from . import pyfrp_geometry
from . import pyfrp_ROI
from . import pyfrp_embryo
from . import pyfrp_molecule



#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""Plotting module for PyFRAP toolbox. 

Contains functions and classes that are often used by PyFRAP toolbox and simplify plot creation and management.

"""

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#numpy
import numpy as np
import scipy.interpolate

#Plotting
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
import matplotlib.pyplot as plt
import matplotlib.patches as ptc

#Misc
import sys
import os

#PyFRAP
import pyfrp_img_module
from pyfrp_term_module import *
from pyfrp.modules import pyfrp_idx_module

#===========================================================================================================================================================================
#Module Functions
#===========================================================================================================================================================================

class FRAPBoundarySelector():
	
	"""Simple GUI widget to select circular FRAP boundaries.
	
	Has useful center marker that is activatable that helps finding the 
	center of the image.
	
	Mouse Input:
		
		* Left: Set center.
		* Right: Set Radius.
		* Middle: Activate center marker.
		
	Keyboard Input:
	
		* Left Arrow: Move center to the left.
		* Right Arrow: Move center to the right.
		* Up Arrow: Move center upwards.
		* Down Arrow: Move center downwards.
		* Control + Up Arrow: Increase circle radius.
		* Control + Down Arrow: Decrease circle radius.
	
	.. note:: If ``embryo`` is given at initiation, will use first image specified
	   in embryo's ``fileList`` as background image.
	   
	Example Usage:
	
	>>> sel=FRAPBoundarySelector(fn="path/to/img/file")

	Use mouse/keyboard to define circular boundary.
	
	>>> center,radius=sel.getResults()
	
	Keyword Args:
		embryo (pyfrp.subclasses.pyfrp_embryo.embryo): PyFRAP embryo instance.
		fn (str): Filepath to image file taken for boundary selection.
		
	"""
	
	def __init__(self,embryo=None,fn=None,img=None):
		
		#Passing embryo to class
		self.embryo=embryo	
		self.fn=fn
		
		#Img
		self.img=img
		
		#Plot resolution
		self.dpi = 100
	
		#Some bookkeeping variables
		self.radiusPt=[]
		self.centerPt=[]
		self.centerMarker=[]
		
		#Results
		self.radius=None
		self.center=None
		
		#Creating figure and canvas
		self.createCanvas()
		
		#Close everthing down if faulty input
		self.checkInput()
			
		#Plot image if existent
		self.showFirstDataImg()
		plt.show()
	
	def checkInput(self):
		
		"""Checks if at least one of the two keyword arguments, ``embryo`` or ``fn``, is given.
		
		If not, prints error message and closes down widget.
		"""
		
		if self.fn==None and self.embryo==None and self.img==None:
			printError("No Embryo, image or fn defined. Going to exit.")
			plt.close(self.fig)
			return
	
	def createCanvas(self):
		
		"""Creates figure and canvas used for plotting."""
			
		h=500/self.dpi
		v=500/self.dpi
		
		self.fig = plt.figure()
		self.fig.show()
		self.fig.set_size_inches(h,v,forward=True)
		
		self.canvas=self.fig.canvas
		
		self.ax = self.fig.add_subplot(111)
		
		if self.embryo!=None:
			self.ax.set_xlim([0, self.embryo.dataResPx])
			self.ax.set_ylim([0, self.embryo.dataResPx])
			
		#Connect to mouseclick event
		self.fig.canvas.mpl_connect('close_event', self.closeFigure)
		self.canvas.mpl_connect('button_press_event', self.getMouseInput)
		self.canvas.mpl_connect('key_press_event', self.keyPressed)
		
		self.canvas.draw()
		
		return 
	
	def keyPressed(self,event):
		
		"""Directs all key press events to the respective functions."""
		
		if event.key=='left':
			self.moveLeft()
		elif event.key=='right':
			self.moveRight()
		elif event.key=='up':
			self.moveUp()
		elif event.key=='down':
			self.moveDown()
		elif event.key=='ctrl+up':
			self.increaseRadius()
		elif event.key=='ctrl+down':
			self.decreaseRadius()
			
	def moveLeft(self):
		
		"""Moves center 1 px to the left."""
		
		if self.center!=None:
			self.center=[self.center[0]-1,self.center[1]]
			self.drawCenter()
		
		
	def moveRight(self):
		
		"""Moves center 1 px to the right."""
		
		if self.center!=None:
			self.center=[self.center[0]+1,self.center[1]]
			self.redraw()
	
	def moveUp(self):
		
		"""Moves center 1 px up."""
		
		if self.center!=None:
			self.center=[self.center[0],self.center[1]+1]
			self.redraw()
	
	def moveDown(self):
		
		"""Moves center 1 px down."""
		
		if self.center!=None:
			self.center=[self.center[0],self.center[1]-1]
			self.redraw()
	
	def redraw(self):
		
		"""Redraws both center and radius if available."""
		
		if self.center!=None:
			self.drawCenter()
		if self.radius!=None:
			self.drawRadius()
			
	def increaseRadius(self):
		
		"""Increases radius by 1 px."""
		
		if self.radius!=None:
			self.radius=self.radius+1
			self.redraw()
			
	def decreaseRadius(self):
		
		"""Decreases radius by 1 px."""
		
		if self.radius!=None:
			self.radius=self.radius-1
			self.redraw()
			
	def closeFigure(self,event):
		
		"""Returns center and radius at close ``event``."""
		
		return self.center,self.radius
		
	def getEmbryo(self):	
		
		"""Returns ``embryo`` object if given.``"""
		
		return self.embryo
	
	def getResults(self):
		
		"""Returns center and radius."""
		
		return self.center,self.radius
	
	def drawCenterMarker(self):
		
		"""Draws a yellow marker in center of the image, making it 
		easier to find image center when selecting center of boundary."""
		
		centerImg=[self.img.shape[0]/2.,self.img.shape[1]/2.]
		
		if len(self.centerMarker)>0:
			self.clearCenterMarker()
		else:	
			pt=ptc.Circle(centerImg,radius=3,fill=True,color='y')
			self.centerMarker.append(self.ax.add_patch(pt))
			
		self.fig.canvas.draw()
		
		return self.centerMarker
		
	def clearCenterMarker(self):
		
		"""Removes center maker from canvas."""
	
		for pt in self.centerMarker:
			pt.remove()
			
		self.fig.canvas.draw()	
			
		self.centerMarker=[]
	
	def drawCenter(self):
		
		"""Draws a red marker at selected center on canvas."""
		
		if len(self.centerPt)>0:
			self.clearCenter()
	
		pt=ptc.Circle(self.center,radius=3,fill=True,color='r')
		self.centerPt.append(self.ax.add_patch(pt))
		
		self.fig.canvas.draw()
		
		return self.centerPt
	
	def clearCenter(self):
	
		"""Removes center marker from canvas."""
		
		for pt in self.centerPt:
			pt.remove()
			
		self.centerPt=[]
		
		self.fig.canvas.draw()
		
	def drawRadius(self):
		
		"""Draws a red circle around selected center with selected radius on canvas."""
		
		if len(self.radiusPt)>0:
			self.clearRadius()
			
		pt=ptc.Circle(self.center,radius=self.radius,fill=False,color='r')
		self.radiusPt.append(self.ax.add_patch(pt))
		
		self.fig.canvas.draw()
		
		return self.radiusPt
	
	def clearRadius(self):
		
		"""Removes circle from canvas."""
		
		for pt in self.radiusPt:
			pt.remove()
			
		self.radiusPt=[]
		
		self.fig.canvas.draw()
		
		return self.radiusPt
		
		
	def showFirstDataImg(self):
		
		"""Shows either first data image defined in ``embryo.fileList`` or 
		image specified by ``fn``.
		
		.. note:: If both are given, will use the embryo option.
		
		"""
		
		if self.embryo!=None:
			self.embryo.updateFileList()
		
			fnImg=self.embryo.fnDatafolder
			if fnImg[-1]!='/':
				fnImg=fnImg+'/'
			fnImg=fnImg+self.embryo.fileList[0]
			
			self.img=pyfrp_img_module.loadImg(fnImg,self.embryo.dataEnc)
		
		elif self.fn!=None:
			self.img=pyfrp_img_module.loadImg(self.fn,'uint16')
			self.ax.set_xlim([0, self.img.shape[0]])
			self.ax.set_ylim([0, self.img.shape[1]])
		
		
		
		self.showImg(self.img)
	
	def showImg(self,img):
		
		"""Shows image on canvas.
		
		Args:
			img (numpy.ndarray): Image to be shown.
		
		"""
		
		self.ax.imshow(img)
		self.fig.canvas.draw()
		
		return self.canvas
	
	def computeRadiusFromCoordinate(self,x,y):
		
		"""Computes radius from given cordinate ``(x,y)``.
		"""
		
		return np.sqrt((x-self.center[0])**2+(y-self.center[1])**2)
		
	def getMouseInput(self,event):
		
		"""Directs mouse input to the right actions."""
		
		#Check if click in axes
		if event.xdata==None:
			return
		
		#Left click to define center
		if event.button==1:
			
			self.center=[event.xdata,event.ydata]
			
			self.drawCenter()
			
			if len(self.radiusPt)>0:
				self.drawRadius()
				
		#Right click to define radius
		elif event.button==3:
			
			if len(self.centerPt)>0:
			
				self.radius=self.computeRadiusFromCoordinate(event.xdata,event.ydata)
				self.drawRadius()
			
		#Middle click to activate center marker	
		if event.button==2:
			
			self.drawCenterMarker()
				
		self.fig.canvas.draw()
		
		return
	
def makeSubplot(size,titles=None,tight=False,sup=None,proj=None,fig=None,show=True):
	
	"""Generates matplotlib figure with (x,y) subplots.
	
	.. note:: List of ``titles`` needs to be same size as desired number of axes.
	   Otherwise will turn off titles.
	
	.. note:: List of ``proj`` needs to be same size as desired number of axes.
	   Otherwise will turn off projections.
	   
	Example:
	
	>>> makeSubplot([2,2],titles=["Axes 1", "Axes 2", "Axes 3", "Axes 4"],proj=[None,None,'3d',None])
	
	will result in
	
	.. image:: ../imgs/pyfrp_plot_module/makeSubplot.png
	
	Args:
		size (list): Size of subplot arrangement.
	
	Keyword Args:
		titles (list): List of axes titles.
		tight (bool): Use tight layout.
		sup (str): Figure title.
		proj (list): List of projections.
		fig (matplotlib.figure): Figure used for axes.
		show (bool): Show figure right away.
	
	Returns:
		tuple: Tuple containing:
			
			* fig (matplotlib.figure): Figure.
			* axes (list): List of Matplotlib axes.
	
	"""
	
	#How many axes need to be created
	n_ax=size[0]*size[1]
	
	if proj==None:
		proj=n_ax*[None]
	
	#Creating figure
	if fig==None:
		fig=plt.figure()
		if show:
			fig.show()
	fig.set_tight_layout(tight)
	
	#Suptitle
	if sup!=None:
		fig.suptitle(sup)
	
	#Add axes
	axes=[]
	for i in range(n_ax):
		
		try:
			if proj[i]!=None:
				ax=fig.add_subplot(size[0],size[1],i+1,projection=proj[i])	
			else:
				ax=fig.add_subplot(size[0],size[1],i+1)
		except IndexError:
			printWarning("Axes " + str(i) + "does not have a projection specified. Will create normal 2D plot.")
			ax=fig.add_subplot(size[0],size[1],i+1)
				
		axes.append(ax)
		
		#Print titles 
		if titles!=None:
			try:
				ax.set_title(titles[i])
			except IndexError:
				printWarning("Axes " + str(i) + " does not have a title specified.")
	#Draw
	plt.draw()
	
	#Return axes handle
	return fig,axes

def makeGeometryPlot(titles=None,tight=False,sup=None,fig=None,show=True,unit="px"):
	
	"""Generates matplotlib figure  and single axes optimized for geometry plots.
	
	See also :py:func:`pyfrp.modules.pyfrp_plot_module.makeSubplot`.
	
	Keyword Args:
		titles (list): List of axes titles.
		tight (bool): Use tight layout.
		sup (str): Figure title.
		fig (matplotlib.figure): Figure used for axes.
		show (bool): Show figure right away.
		unit (str): Unit displayed in axes label.
	
	Returns:
		tuple: Tuple containing:
			
			* fig (matplotlib.figure): Figure.
			* axes (list): List of Matplotlib axes.
	
	"""
	
	fig,axes=makeSubplot([1,1],proj=['3d'],titles=titles,tight=tight,sup=sup,fig=fig,show=show)
	
	if unit==None:
		axes[0].set_xlabel("x")
		axes[0].set_ylabel("y")
		axes[0].set_zlabel("z")
	else:	
		axes[0].set_xlabel("x ("+unit+")")
		axes[0].set_ylabel("y ("+unit+")")
		axes[0].set_zlabel("z ("+unit+")")
	
	return fig,axes
	
	

def adjustImshowRange(axes,vmin=None,vmax=None):
	
	"""Adjust display range of ``matplotlib.pyplot.imshow`` plots in 
	list of axes.
	
	Finds first image artist in each axes in ``axes`` list and then
	sets display range to ``[vmin,vmax]``.
	
	Args:
		axes (list): List of matplotlib axes.
		
	Keyword Args:
		vmin (float): Minimum value of display range.
		vmax (float): Maximum value of display range.
		
	Returns:
		 list: Updated list of matplotlib axes.
	
	"""
	
	#Loop through axes
	for ax in axes:
		
		#Grab plot
		implot=findArtist(ax,"Image")
		
		#Rescale
		if implot!=None:
			implot.set_clim(vmin,vmax)
		
	#Draw
	redraw(ax)
		
	return axes

def findArtist(ax,key):
	
	"""Finds ``matplotlib.artist`` which name contains ``key``.
	
	.. note:: Will stop searching after first artist is found.
	
	Will return ``None`` if no artist can be found.
	
	Args:
		ax (matplotlib.axes): Matplotlib axes.
		key (str): Key used for search.
		
	Returns:
		matplotlib.artist: Matplotlib artist.
	
	"""
	
	c=ax.get_children()
	
	for x in c:
		if key in str(x):
			return x
			break
		
	return None	

def redraw(ax):
	
	"""Redraws axes's figure's canvas.
	
	Makes sure that current axes content is visible.
	
	Args:
		ax (matplotlib.axes): Matplotlib axes.
		
	Returns:
		matplotlib.axes: Matplotlib axes 
	
	"""
			
	ax.get_figure().canvas.draw()
	
	return ax

def plotTS(xvec,yvec,label='',title='',sup='',ax=None,color=None,linewidth=1,legend=True,linestyle='-',show=True,alpha=1.,legLoc=-1):
	
	"""Plot timeseries all-in-one function.
		
	Args:
		xvec (numpy.ndarray): x-data to be plotted.
		yvec (numpy.ndarray): y-data to be plotted.
	
	Keyword Args:
		ax (matplotlib.axes): Matplotlib axes used for plotting. If not specified, will generate new one.
		color (str): Color of plot.
		linestyle (str): Linestyle of plot.
		linewidth (float): Linewidth of plot.
		legend (bool): Show legend.
		sup (str): Figure title.
		title (str): Axes title.
		label (str): Label for legend.
		show (bool): Show figure.
		alpha (float): Transparency of line.
		legLoc (int): Location of legend.
	
	Returns:
		matplotlib.axes: Axes used for plotting.
	
	"""
	
	if len(xvec)!=len(yvec):
		printWarning('len(xvec) != len (yvec). This could be due to incomplete simulation/analysis/pinning. Will not plot.')
		return None
	
	if ax==None:
		fig,axes = makeSubplot([1,1],titles=[title],sup=sup,tight=False,show=show)
		ax=axes[0]
	else:
		ax.set_title(title)
		
	ax.plot(xvec,yvec,color=color,label=label,linestyle=linestyle,alpha=alpha)
	
	if legend:
		if legLoc==-1:
			ax.get_figure().subplots_adjust(right=0.7)
			ax.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.)
		else:
			ax.legend(loc=legLoc)
		
	redraw(ax)
	
	return ax

def plotSolutionVariable(x,y,val,ax=None,vmin=None,vmax=None,nlevels=25,colorbar=True,plane='xy',zs=None,zdir=None,title="Solution Variable",sup="",dThresh=None,nPts=1000,mode='normal',typ='contour'):
		
	"""Plots simulation solution variable as 2D contour plot or 3D surface plot.
	
	.. note:: If no ``ax`` is given, will create new one.
	
	.. note:: ``x`` and ``y`` do not necessarily have to be coordinates in x/y-direction, but rather correspond to 
	   the two directions defined in ``plane``. 
	
	``plane`` variable controls in which plane the solution variable is supposed to be plotted. 
	Acceptable input variables are ``"xy","xz","yz"``. See also
	:py:func:`pyfrp.subclasses.pyfrp_ROI.ROI.getMaxExtendPlane`.
	
	``mode`` controls which contour/surface function is used:
	
		* ``normal``: Will create rectangular grid and use ``scipy.interpolate.gridddata`` and interpolate solution onto it,
		  then plot using ``matplotlib.pyplot.contourf``, see also http://matplotlib.org/api/pyplot_api.html#matplotlib.pyplot.contourf and
		  http://docs.scipy.org/doc/scipy-0.16.0/reference/generated/scipy.interpolate.griddata.html#scipy.interpolate.griddata .
		
		* ``tri``: Will plot irregular grid using ``matplotlib.pyplot.tricontourf``. See also 
		  http://matplotlib.org/api/pyplot_api.html#matplotlib.pyplot.tricontourf .
	
	``typ`` controls which plot type is used:
	
		* ``contour``: Produces contour plots using either ``matplotlib.pyplot.contourf`` or ``matplotlib.pyplot.tricontourf``.
		
		* ``surface``: Produces contour plots using either ``matplotlib.Axed3D.plot_surface`` or ``matplotlib.pyplot.plot_trisurf``.
	
	.. warning:: ``matplotlib.pyplot.tricontourf`` has problems when ``val`` only is in a single level of contour plot.
		To avoid this, we currently add some noise in this case just to make it plottable. This is not the most elegant
		solution. (only in case of 3D plotting)
	
	Args:
		x (numpy.ndarray): x-coordinates.
		y (numpy.ndarray): y-coordinates.
		val (numpy.ndarray): Solution variable values.
		
	Keyword Args:
		ax (matplotlib.axes): Axes used for plotting.
		vmin (float): Minimum value displayed in contour plot.
		vmax (float): Maximum value displayed in contour plot.
		nlevels (int): Number of contour levels.
		colorbar (bool): Display color bar.
		plane (str): Plane in which solution variable is supposed to be plotted.
		zs (float): In case of a 3D plot, height in direction zdir where to put contour.
		zdir (str): Orthogonal direction to plane.
		nPts (int): Number of points used for interpolating (only if ``mode=normal``).
		mode (str): Which contour function to use.
		title (str): Title of plot.
		typ (str): Type of plot.
		
	Returns:
		matplotlib.axes: Axes used for plotting.
	
	"""
	
	#Check of entered plot type makes sense
	if typ not in ['contour','surface']:
		printError("Unknown plot type "+ typ)
		return ax
	
	#Make axes if necessary
	if ax==None:
		if (zs!=None and zdir!=None) or typ=='surface':
			fig,axes = makeSubplot([1,1],titles=[title],sup=sup,proj=['3d'])
		else:	
			fig,axes = makeSubplot([1,1],titles=[title],sup=sup)
		
		ax=axes[0]
	else:
		ax.set_title(title)
		
	
	#vmin/vmax/levels
	vmin,vmax,levels=makeFittingLevels(vmin,vmax,val,nlevels)
	
	#Make grid
	grid = np.meshgrid(np.linspace(min(x), max(x), nPts),np.linspace(min(y), max(y), nPts))
	
	#Interpolate
	xy=np.vstack((x,y))
	valPlot = scipy.interpolate.griddata(xy.T, val, tuple(grid), 'linear')
	
	#If dThresh is given, filter all nodes that are further apart from their neighbor than dThresh
	if dThresh!=None:
		idxs=pyfrp_idx_module.maskMeshByDistance(x,y,dThresh,grid)
		valPlot[idxs]=np.nan
		
	#Stupid fix for tricontourf 3D problem
	if zs!=None and zdir!=None and mode=='tri':
		if min(val)==max(val):
			
			"""IDEA:
			Add a little bit of noise to make it work. Not best solution ever. Needs to be changed.
			"""

			add=(1.01*vmax-vmin)/(nlevels)*np.random.randn(np.shape(val)[0])
			val=val+add
	
	#Plot 
	if mode=='tri':
		if typ=='surface':
			solPlot=ax.plot_trisurf(x,y,val,cmap='jet',vmin=vmin,vmax=vmax)
		elif typ=='contour':
			solPlot=ax.tricontourf(x,y,val,vmin=vmin,vmax=vmax,levels=levels,offset=zs,zdir=zdir,extend='both')
	else:
		
		if typ=='surface':
			solPlot=ax.plot_surface(grid[0],grid[1],valPlot,cmap='jet',vmin=vmin,vmax=vmax)
		elif typ=='contour':
			solPlot=ax.contourf(grid[0],grid[1],valPlot,vmin=vmin,vmax=vmax,levels=levels,offset=zs,zdir=zdir)	
	
	#Label
	if len(plane)!=2:
		printError("Don't understand plane="+plane+". Will not plot.")
		return ax	
	else:
		ax.set_xlabel(plane[0])
		ax.set_ylabel(plane[1])

	ax.autoscale(enable=True, axis='both', tight=True)
	
	if colorbar:
		cb=plt.colorbar(solPlot,orientation='horizontal',pad=0.05)
	
	#plt.draw()
	ax.get_figure().canvas.draw()
	
	return ax	

def makeFittingLevels(vmin,vmax,val,nlevels,buff=0.01):
	
	"""Generates array with fitting levels for contour plots.
	
	.. note:: If ``vmin=None`` or ``vmax=None``, will pick the minimum/maximum 
	   value of ``val``.
	
	Args:
		val (numpy.ndarray): Array to be plotted.
		vmin (float): Minimum value displayed in contour plot.
		vmax (float): Maximum value displayed in contour plot.
		nlevels (int): Number of contour levels.
		
	Keyword Args:	
		buff (float): Percentage buffer to be added to both sides of the array.
		
	Returns:
		tuple: Tuple containing:
		
			* vmin (float): Minimum value displayed in contour plot.
			* vmax (float): Maximum value displayed in contour plot.
			* levels (numpy.ndarray): Level array to be handed to contour function.
	
	"""
	
	
	if vmin==None:
		vmin=min(val)
	if vmax==None:
		vmax=max(val)
		
	levels=np.linspace((1-buff)*vmin,(1+buff)*vmax,nlevels)
	
	return vmin,vmax,levels
	
def set3DAxesEqual(ax):

	"""Make axes of 3D plot have equal scale.
	
	This is one possible solution to Matplotlib's
	ax.set_aspect('equal') and ax.axis('equal') not working for 3D.
	
	Modified from http://stackoverflow.com/questions/13685386/matplotlib-equal-unit-length-with-equal-aspect-ratio-z-axis-is-not-equal-to .
	
	Args:
		ax (matplotlib.axes): A matplotlib axes.
		
	Returns:
		matplotlib.axes: Modified matplotlib axes.
		
	"""

	xLimits = ax.get_xlim3d()
	yLimits = ax.get_ylim3d()
	zLimits = ax.get_zlim3d()

	xRange = abs(xLimits[1] - xLimits[0])
	xMiddle = np.mean(xLimits)
	yRange = abs(yLimits[1] - yLimits[0])
	yMiddle = np.mean(yLimits)
	zRange = abs(zLimits[1] - zLimits[0])
	zMiddle = np.mean(zLimits)

	# The plot bounding box is a sphere in the sense of the infinity
	# norm, hence I call half the max range the plot radius.
	plotRadius = 0.5*max([xRange, yRange, zRange])

	ax.set_xlim3d([xMiddle - plotRadius, xMiddle + plotRadius])
	ax.set_ylim3d([yMiddle - plotRadius, yMiddle + plotRadius])
	ax.set_zlim3d([zMiddle - plotRadius, zMiddle + plotRadius])
	
	return ax

def getPubParms(fontSize=10,labelFontSize=10,tickFontSize=10,legendFontSize=10):
	
	"""Returns dictionary with good parameters for nice 
	publication figures.
	
	Resulting ``dict`` can be loaded via ``plt.rcParams.update()``.
	
	.. note:: Use this if you want to include LaTeX symbols in the figure.
	
	Keyword Args:
		fontSize (int): Font size.
		labelFontSize (int): Font size.
		tickFontSize (int): Font size.
		legendFontSize (int): Font size.
		
	Returns:
		dict: Parameter dictionary.
	
	"""
	
	
	params = {'backend': 'ps',
		'font.size': fontSize,
		'axes.labelsize': labelFontSize,
		'legend.fontsize': legendFontSize,
		'xtick.labelsize': tickFontSize,
		'ytick.labelsize': tickFontSize,
		'text.usetex': True,
		'font.family': 'sans-serif',
		'font.sans-serif': 'Bitstream Vera Sans, Lucida Grande, Verdana, Geneva, Lucid, Arial, Helvetica, Avant Garde, sans-serif',
		#'ytick.direction': 'out',
		'text.latex.preamble': [r'\usepackage{helvet}', r'\usepackage{sansmath}',r'\usepackage{siunitx}'] , #r'\sansmath',    
		'ytick.direction' : 'out',
		'xtick.direction' : 'out'
		}
		
	return params

def turnAxesForPub(ax,adjustFigSize=True,figWidthPt=180.4,figHeightPt=None,ptPerInches=72.27,fontSize=10,labelFontSize=10,tickFontSize=10,legendFontSize=10):
	
	"""Turns axes nice for publication.
	
	If ``adjustFigSize=True``, will also adjust the size the figure. 
	
	Args:
		ax (matplotlib.axes): A matplotlib axes.
		
	Keyword Args:
		adjustFigSize (bool): Adjust the size of the figure.
		figWidthPt (float): Width of the figure in pt.
		figHeightPt (float): Height of the figure in pt.
		ptPerInches (float): Resolution in pt/inches.
		fontSize (int): Font size.
		labelFontSize (int): Font size.
		tickFontSize (int): Font size.
		legendFontSize (int): Font size.
		
	Returns:
		matplotlib.axes: Modified matplotlib axes.
		
	"""

	params=getPubParms(fontSize=fontSize,labelFontSize=labelFontSize,tickFontSize=tickFontSize,legendFontSize=legendFontSize)
	plt.rcParams.update(params)
	
	ax=setPubAxis(ax)
	
	setPubFigSize(ax.get_figure(),figWidthPt=figWidthPt,figHeightPt=figHeightPt,ptPerInches=ptPerInches)
	
	ax=closerLabels(ax,padx=3,pady=1)
	
	return ax
		
def setPubAxis(ax):		
	
	"""Gets rid of top and right axis.
	
	Args:
		ax (matplotlib.axes): A matplotlib axes.
		
	Returns:
		matplotlib.axes: Modified matplotlib axes.
	
	"""
	
	ax.spines['top'].set_color('none')
	ax.spines['right'].set_color('none')
	ax.xaxis.set_ticks_position('bottom')
	ax.yaxis.set_ticks_position('left')
	
	ax.spines["left"].axis.axes.tick_params(direction="outward") 
	ax.spines["bottom"].axis.axes.tick_params(direction="outward") 
	
	return ax
		
def setPubFigSize(fig,figWidthPt=180.4,figHeightPt=None,ptPerInches=72.27):
	
	"""Adjusts figure size/aspect.
	
	If ``figHeightPt`` is not given, will use golden ratio to
	compute it.
	
	Keyword Args:
		figWidthPt (float): Width of the figure in pt.
		figHeightPt (float): Height of the figure in pt.
		ptPerInches (float): Resolution in pt/inches.
	
	Returns:
		matplotlib.figure: Adjust figure.
	
	"""
	
	inchesPerPt = 1.0/ptPerInches
	goldenMean = (np.sqrt(5)-1.0)/2.0         # Aesthetic ratio
	figWidth = figWidthPt*inchesPerPt  # width in inches
	if figHeightPt==None:
		figHeight = figWidth*goldenMean      # height in inches
	else:
		figHeight = figHeightPt*inchesPerPt
	figSize =  [figWidth,figHeight]
	
	fig.set_size_inches(figSize[0],figSize[1])
	
	fig.subplots_adjust(bottom=0.25)
	fig.subplots_adjust(left=0.2)
	fig.subplots_adjust(top=0.9)
	
	return fig

def closerLabels(ax,padx=10,pady=10):
	
	"""Moves x/y labels closer to axis."""
	
	ax.xaxis.labelpad = padx
	ax.yaxis.labelpad = pady
	return ax

def getRandomColor():
	
	"""Returns triplet defining a random color.
	"""
	
	return np.random.rand(3,1)

def is3DAxes(ax):
	
	"""Returns if an axes is a 3D axis.
	
	Args:
		ax (matplotlib.axes): A matplotlib axes.
		
	Returns:
		bool: True if 3d axis.
	"""
	
	if hasattr(ax, 'get_zlim'): 
		return True
	else:
		return False




#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

#Module containing basic custom PyQT classes:
#1) basicCanvasDialog



#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#Misc
import sys
import time
import os, os.path

#PyFRAP modules
from pyfrp.modules.pyfrp_term_module import *

#PyQT
from PyQt4 import QtGui, QtCore

#matplotlib
import matplotlib.pyplot as plt
import matplotlib
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas

"""
Apparently the NavigationToolbar naming has changed in newer matplotlib versions, thus
we need to test out some cases.
"""

try:
	from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as  NavigationToolbar
except ImportError:
	try:
		from matplotlib.backends.backend_qt4agg import NavigationToolbar2QT as  NavigationToolbar
	except ImportError:
		printWarning("Cannot import NavigationToolbar.")
		
from matplotlib.figure import Figure
	
#===================================================================================================================================
#Basic Dialog with space for QLEs/Btns on left hand side and canvas on right hand side
#===================================================================================================================================

class basicCanvasDialog(QtGui.QDialog):
	
	def __init__(self,parent,xlim=[0,512],ylim=[0,512]):
		
		super(basicCanvasDialog,self).__init__(parent)
				
		self.dpi = 100
		self.setMinimumSize(1000,500) 
		self.resize(1300,500)
		
		#-------------------------------------------------------------------------------------------------------------------
		#Bookkeeping variables
		#-------------------------------------------------------------------------------------------------------------------
		
		self.artists=[]
		
		#-------------------------------------------------------------------------------------------------------------------
		#Buttons
		#-------------------------------------------------------------------------------------------------------------------
		
		#Done button
		self.btnDone=QtGui.QPushButton('Done')
		self.btnDone.connect(self.btnDone, QtCore.SIGNAL('clicked()'), self.donePressed)
	
		#-------------------------------------------------------------------------------------------------------------------
		#Plot frame
		#-------------------------------------------------------------------------------------------------------------------
		
		self.plotFrame = QtGui.QWidget()
		self.plotFrame.setMaximumWidth(1)
		
		#-------------------------------------------------------------------------------------------------------------------
		#Layout
		#-------------------------------------------------------------------------------------------------------------------
		
		self.grid = QtGui.QGridLayout()		
		self.grid.setColumnMinimumWidth(2,200) 
		
		#-------------------------------------------------------------------------------------------------------------------
		#Create Canvas
		#-------------------------------------------------------------------------------------------------------------------
		
		self.createCanvas(xlim=xlim,ylim=ylim)
		
		#-------------------------------------------------------------------------------------------------------------------
		#Final Layout
		#-------------------------------------------------------------------------------------------------------------------
		
		self.vbox = QtGui.QVBoxLayout()
		self.vbox.addWidget(self.canvas)
		self.vbox.addWidget(self.btnDone)
		
		#Add everything to Horizontal Box
		self.hbox = QtGui.QHBoxLayout()
		self.hbox.addLayout(self.grid)
		self.hbox.addLayout(self.vbox)
		
		self.setLayout(self.hbox)    
			
		self.setWindowTitle('basicCanvasDialog')    
		self.show()
	
	def createCanvas(self,xlim=None,ylim=None):
			
		h=500/self.dpi
		v=500/self.dpi
		
		self.fig = Figure( dpi=self.dpi)
		self.fig.set_size_inches(h,v,forward=True)
		self.canvas = FigureCanvas(self.fig)
		self.canvas.setParent(self.plotFrame)
		
		self.ax = self.fig.add_subplot(111)
		
		if xlim!=None:
			self.ax.set_xlim(xlim)
		if ylim!=None:
			self.ax.set_ylim(ylim)
		
		self.canvas.draw()
		#self.plotFrame.adjustSize()
		
		return 
		
	def showImg(self,img):
		
		self.ax.imshow(img)
		self.ax.set_xlim([1,img.shape[0]])
		self.ax.set_ylim([1,img.shape[1]])
		
		self.canvas.draw()
		
		return self.canvas
	
	def connectCanvas(self):
		self.canvas.mpl_connect('button_press_event', self.getMouseCanvas)
		self.canvas.mpl_connect('key_press_event', self.keyPressed)
		
		self.canvas.setFocusPolicy( QtCore.Qt.ClickFocus )
		self.canvas.setFocus()
		
		
	def keyPressed(self):
		printWarning("No Key-Press Action defined.")
	
	def removeArtist(self,idx=-1):
		if len(self.artists)>0:
			self.artists[idx].remove()
			self.artists.pop(idx)
			self.canvas.draw()
		return self.artists
	
	def replaceArtist(self,idx,newArtist):
		if len(self.artists)>idx:
			self.removeArtist(idx=idx)
			self.artists.insert(idx,newArtist)
			self.canvas.draw()
		else:
			self.artists.append(newArtist)
		
		return self.artists
	
	def setArtistColor(self,color):
		for artist in self.artists:
			
			artist.set_color(color)
		self.canvas.draw()	
		return self.artists
	
	def donePressed(self):
		self.done(1)
		return 
	
#===================================================================================================================================
#Basic Dialog for settings of any kind.
#===================================================================================================================================

class basicSettingsDialog(QtGui.QDialog):
	
	def __init__(self,parent):
		
		super(basicSettingsDialog,self).__init__(parent)
				
		self.setMinimumSize(500,500) 
		self.resize(700,500)
		
		#-------------------------------------------------------------------------------------------------------------------
		#Buttons
		#-------------------------------------------------------------------------------------------------------------------
		
		#Done button
		self.btnDone=QtGui.QPushButton('Done')
		self.btnDone.connect(self.btnDone, QtCore.SIGNAL('clicked()'), self.donePressed)
	
		#-------------------------------------------------------------------------------------------------------------------
		#Layout
		#-------------------------------------------------------------------------------------------------------------------
		
		self.grid = QtGui.QGridLayout()		
		self.grid.setColumnMinimumWidth(2,20) 
		
		#-------------------------------------------------------------------------------------------------------------------
		#Validators
		#-------------------------------------------------------------------------------------------------------------------
		
		self.doubleValid=QtGui.QDoubleValidator()
		self.intValid=QtGui.QIntValidator()
		
		#-------------------------------------------------------------------------------------------------------------------
		#Final Layout
		#-------------------------------------------------------------------------------------------------------------------
		
		self.vbox = QtGui.QVBoxLayout()
		self.vbox.addLayout(self.grid)
		
		self.hbox = QtGui.QHBoxLayout()
		self.hbox.addWidget(self.btnDone,stretch=0,alignment=QtCore.Qt.AlignRight)	
		self.vbox.addLayout(self.hbox)
			
		self.setLayout(self.vbox)    
			
		self.setWindowTitle('basicSettingsDialog')    

	def donePressed(self):
		self.done(1)
		return

#===================================================================================================================================
#Basic Selector for a single item out of a list
#===================================================================================================================================

class basicSelectorDialog(QtGui.QDialog):
	
	def __init__(self,List,parent):
		
		super(basicSelectorDialog,self).__init__(parent)
		
		self.item=None
		self.List=List
		
		#Done button
		self.btnDone=QtGui.QPushButton('Done')
		self.btnDone.connect(self.btnDone, QtCore.SIGNAL('clicked()'), self.donePressed)
			
		#QTreeWidget
		self.ListWidget=QtGui.QTreeWidget()
		self.ListWidget.setColumnWidth(0,100)
		self.ListWidget.itemClicked.connect(self.itemClicked)
		self.updateList()
		
		#Layout
		self.vbox = QtGui.QVBoxLayout()
		self.vbox.addWidget(self.ListWidget)
		
		self.hbox = QtGui.QHBoxLayout()
		self.hbox.addWidget(self.btnDone,stretch=0,alignment=QtCore.Qt.AlignRight)	
		self.vbox.addLayout(self.hbox)
			
		self.setLayout(self.vbox)    
		
		self.setWindowTitle('basicSelectorDialog')    
		self.show()
		
	def updateList(self):
		
		self.ListWidget.clear()
		for r in self.List:
			QtGui.QTreeWidgetItem(self.ListWidget,[r])
		return 
	
	def itemClicked(self):
		idx=self.ListWidget.indexFromItem(self.ListWidget.currentItem()).row()
		self.item=self.List[idx]
		
	def getItem(self):
		return self.item
			
	def donePressed(self):
		self.done(1)
		return		

#===================================================================================================================================
#Basic Selector for a list of items out of a list
#===================================================================================================================================

class listSelectorDialog(QtGui.QDialog):
	
	def __init__(self,parent,List,leftTitle="",rightTitle="",itemsRight=[]):
		super(listSelectorDialog,self).__init__(parent)
		#print type(self), type(parent)
		
		#QtGui.QDialog.__init__()
		
		
		self.itemsRight=itemsRight
		self.itemsLeft=list(List)
		
		self.List=List
		
		#Buttons
		self.btnAdd=QtGui.QToolButton()
		self.btnAdd.connect(self.btnAdd, QtCore.SIGNAL('clicked()'), self.addItem)
		self.btnAdd.setArrowType(QtCore.Qt.RightArrow)
		
		self.btnRemove=QtGui.QToolButton()
		self.btnRemove.connect(self.btnRemove, QtCore.SIGNAL('clicked()'), self.removeItem)
		self.btnRemove.setArrowType(QtCore.Qt.LeftArrow)
		
		self.btnDone=QtGui.QPushButton('Done')
		self.btnDone.connect(self.btnDone, QtCore.SIGNAL('clicked()'), self.donePressed)
		
		#Left QtreeWidgetItem
		self.leftList=QtGui.QTreeWidget()
		self.leftList.setHeaderLabels([leftTitle])
		self.leftList.setColumnWidth(0,200)
		self.leftList.setColumnWidth(1,75)
		self.leftList.itemDoubleClicked.connect(self.addItem)
		
		#right QtreeWidgetItem
		self.rightList=QtGui.QTreeWidget()
		self.rightList.setHeaderLabels([rightTitle])
		self.rightList.setColumnWidth(0,200)
		self.rightList.setColumnWidth(1,75)
		self.rightList.itemDoubleClicked.connect(self.removeItem)
		
		#Layout
		self.vbox = QtGui.QVBoxLayout()
		self.vbox.addWidget(self.btnAdd)
		self.vbox.addWidget(self.btnRemove)
		
		self.hbox = QtGui.QHBoxLayout()
		self.hbox.addWidget(self.leftList)
		self.hbox.addLayout(self.vbox)
		self.hbox.addWidget(self.rightList)
		
		self.vbox2 = QtGui.QVBoxLayout()
		self.vbox2.addLayout(self.hbox)
		self.vbox2.addWidget(self.btnDone)
		
		#Init lists
		self.initLeftList()
		self.initRightList()
		
		self.resize(400,500)
		self.setLayout(self.vbox2)
		self.setWindowTitle("list Selector Dialog")
		
		self.show()
	
	def getListDifference(self):
		
		for item in self.itemsLeft:
			if item in self.itemsRight:
				self.itemsLeft.remove(item)
		
		return self.itemsLeft
				
	
	def initLeftList(self):
		
		self.getListDifference()
		
		for item in self.itemsLeft:
			QtGui.QTreeWidgetItem(self.leftList,[item])
			
	def initRightList(self):
		
		for item in self.itemsRight:
			QtGui.QTreeWidgetItem(self.rightList,[item])
				
	def addItem(self):

		#Determine selected item
		self.currentItem=str(self.leftList.currentItem().data(0,0).toString())
		
		#Insert new node in right list
		newNode=QtGui.QTreeWidgetItem(self.rightList,[self.currentItem])
		
		#Remove node in left list
		self.currLeftInd=self.leftList.indexFromItem(self.leftList.currentItem()).row()
		self.leftList.takeTopLevelItem(self.currLeftInd)
		
		self.itemsRight.append(self.currentItem)
		self.itemsLeft.remove(self.currentItem)
		
		
	def removeItem(self):
	
		#Determine selected item
		self.currentItem=str(self.rightList.currentItem().data(0,0).toString())
		
		#Insert new node in left list
		newNode=QtGui.QTreeWidgetItem(self.leftList,[self.currentItem])
		
		#Remove node in right list
		self.currRightInd=self.rightList.indexFromItem(self.rightList.currentItem()).row()
		self.rightList.takeTopLevelItem(self.currRightInd)
		
		self.itemsRight.remove(self.currentItem)
		self.itemsLeft.append(self.currentItem)

	def getSelection(self):
		return self.itemsRight
	
	def donePressed(self):
		self.done(1)
		return


#===================================================================================================================================
#Basic Selector for a list of items out of a list
#===================================================================================================================================

class advancedListSelectorDialog(listSelectorDialog):
	
	def __init__(self,parent,List,leftTitle="",rightTitle="",itemsRight=[]):
		
		super(advancedListSelectorDialog,self).__init__(parent,List,leftTitle=leftTitle,rightTitle=rightTitle)
		#print type(self),type(parent)
		#raw_input()
		
		#listSelectorDialog.__init__(parent,List,leftTitle=leftTitle,rightTitle=rightTitle,itemsRight=itemsRight)
		
		self.btnUp=QtGui.QToolButton()
		self.btnUp.connect(self.btnUp, QtCore.SIGNAL('clicked()'), self.upItem)
		self.btnUp.setArrowType(QtCore.Qt.UpArrow)
		
		self.btnDown=QtGui.QToolButton()
		self.btnDown.connect(self.btnDown, QtCore.SIGNAL('clicked()'), self.downItem)
		self.btnDown.setArrowType(QtCore.Qt.DownArrow)
		
		self.vbox.addWidget(self.btnUp)
		self.vbox.addWidget(self.btnDown)
		
		self.setWindowTitle("advanced List Selector Dialog")
		
	def upItem(self):
		
		#Determine selected item
		self.currentItem=str(self.rightList.currentItem().data(0,0).toString())
		
		#Determine index in sel prop list
		ind=self.itemsRight.index(self.currentItem)
		
		#Swap in list
		if ind>0:
			self.itemsRight[ind-1], self.itemsRight[ind] = self.itemsRight[ind], self.itemsRight[ind-1]
		
		#Clear list and recreate it
		self.rightList.clear()
		self.initRightList()
		self.currentRightItem=self.rightList.topLevelItem(ind-1)
		self.rightList.setCurrentItem(self.currentRightItem)
		
	def downItem(self):
		
		#Determine selected item
		self.currentItem=str(self.rightList.currentItem().data(0,0).toString())
		
		#Determine index in sel prop list
		ind=self.itemsRight.index(self.currentItem)
		
		#Swap in list
		if ind<len(self.itemsRight)-1:
			self.itemsRight[ind+1], self.itemsRight[ind] = self.itemsRight[ind], self.itemsRight[ind+1]
		
		#Clear list and recreate it
		self.rightList.clear()
		self.initRightList()
		self.currentRightItem=self.rightList.topLevelItem(ind+1)
		self.rightList.setCurrentItem(self.currentRightItem)	
		
#===================================================================================================================================
#Basic Progress Dialog
#===================================================================================================================================
		
class progressDialog(QtGui.QDialog):
	
	def __init__(self,parent):
		super(progressDialog,self).__init__(parent)
		
		#Labels
		self.lblName = QtGui.QLabel("Something in progress...", self)
		
		#Buttons
		self.btnCancel=QtGui.QPushButton('Cancel')
		self.btnCancel.connect(self.btnCancel, QtCore.SIGNAL('clicked()'), self.cancel)	
		
		#ProgressBar
		self.progressbar = QtGui.QProgressBar()
		self.progressbar.setMinimum(1)
		self.progressbar.setMaximum(100)
		
		#Layout
		self.vbox = QtGui.QVBoxLayout()
		self.vbox.addWidget(self.lblName)
		self.vbox.addWidget(self.progressbar)
		self.vbox.addWidget(self.btnCancel)
		
		self.setLayout(self.vbox)
		self.setWindowTitle('Progress Dialog')
		
		self.show()	
	
	def cancel(self):
	
		self.accepted.emit()

#===================================================================================================================================
#Basic PyFRAP Thread
#===================================================================================================================================

#Simple worker class        
class pyfrpWorker(QtCore.QObject):
	
	taskFinished = QtCore.pyqtSignal()
	start = QtCore.pyqtSignal()
	
	def __init__(self, function, *args, **kwargs):
		super(pyfrpWorker, self).__init__()
		
		self.function = function
		self.args = args
		self.kwargs = kwargs
		self.start.connect(self.run)
		
	#@QtCore.pyqtSlot() (Interesting: the pyqtslot decorator seems to block the start signal...)
	def run(self):
		self.function(*self.args, **self.kwargs)
		self.taskFinished.emit()
		
		
class pyfrpThread(QtCore.QThread):
	
	progressSignal = QtCore.pyqtSignal(int)
	
	def __init__(self, parent=None):
		QtCore.QThread.__init__(self)
		
	def __del__(self):
		self.wait()
    
		
#===================================================================================================================================
#Basic Wait Dialog
#===================================================================================================================================
		
class waitDialog(QtGui.QDialog):
	
	def __init__(self,parent):
		super(waitDialog,self).__init__(parent)
		
		#Labels
		self.lblName = QtGui.QLabel("Something in progress...", self)
		
		#Buttons
		self.btnCancel=QtGui.QPushButton('Cancel')
		self.btnCancel.connect(self.btnCancel, QtCore.SIGNAL('clicked()'), self.cancel)	
		
		#Layout
		self.vbox = QtGui.QVBoxLayout()
		self.vbox.addWidget(self.lblName)
		self.vbox.addWidget(self.btnCancel)
		
		self.setLayout(self.vbox)
		self.setWindowTitle('Progress Dialog')
		
		self.show()	
	
	def cancel(self):
		self.accepted.emit()
	
	


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""PyFRAP module for running Gmsh on .geo files. Module mainly has the following features:

	* Functions for updating parameters in standard .geo files.
	* Mesh refinement.
	* Running Gmsh

This module together with pyfrp.pyfrp_gmsh_geometry and pyfrp.pyfrp_gmsh_IO_module works partially as a python gmsh wrapper, however is incomplete.
If you want to know more about gmsh, go to http://gmsh.info/doc/texinfo/gmsh.html .
	
"""

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#Numpy/Scipy
from numpy import *

#Misc
import os
import shutil
from tempfile import mkstemp
import subprocess
import time
import shlex
import platform

#PyFRAP
import pyfrp_gmsh_IO_module
import pyfrp_misc_module
from pyfrp_term_module import *

                   
#===========================================================================================================================================================================
#Module Functions
#===========================================================================================================================================================================

def updateCylinderGeo(fn,radius,height,center,run=True,debug=False):
	
	"""Upates parameters in default *cylinder.geo* file.
	
	.. note:: Debug will also activate full debugging output of gmsh. See also
	   http://gmsh.info/doc/texinfo/gmsh.html#Command_002dline-options .
	
	.. note:: For function to work, parameters in geo file need to be defined as follows:
	   
		* radius -> radius
		* height -> height
		* center -> center_x , center_y
	
	Args:
		fn (str): Filepath.
		radius (float): New cylinder radius.
		height (float): New cylinder height.
		center (list): New cylinder center.
	
	Keyword Args:
		run (bool): Run gmsh on updated file.
		debug (bool): Print debugging messages.
		
	Returns:
		str: Path to mesh file.
		
	""" 
	
	v=5*int(debug)
	
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"radius",radius)
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"height",height)
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"center_x",center[0])
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"center_y",center[1])

	if run:
		gmshBin=getGmshBin()
		os.system(gmshBin + "  -v " + str(v) +" -3 " + fn)
	
	fn_msh=fn.replace(".geo",".msh")
	
	return fn_msh

def updateConeGeo(fn,upperRadius,lowerRadius,height,center,run=True,debug=False):
	
	"""Upates parameters in default *cone.geo* file.
	
	.. note:: Debug will also activate full debugging output of gmsh. See also
	   http://gmsh.info/doc/texinfo/gmsh.html#Command_002dline-options .
	
	.. note:: For function to work, parameters in geo file need to be defined as follows:
	   
		* upperRadius -> upper_radius
		* lowerRadius -> lower_radius
		* slice_height -> slice_height
		* center -> center_x , center_y
	
	Args:
		fn (str): Filepath.
		upperRadius (float): New upper cone radius.
		lowerRadius (float): New lower cone radius.
		height (float): New cone height.
		center (list): New cone center.
	
	Keyword Args:
		run (bool): Run gmsh on updated file.
		debug (bool): Print debugging messages.
		
	Returns:
		str: Path to mesh file.
		
	""" 
	
	v=5*int(debug)
	
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"upper_radius",upperRadius)
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"lower_radius",lowerRadius)
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"height",height)
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"center_x",center[0])
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"center_y",center[1])

	if run:
		gmshBin=getGmshBin()
		os.system(gmshBin + "  -v " + str(v) +" -3 " + fn)
	
	fn_msh=fn.replace(".geo",".msh")
	
	return fn_msh


def updateBallGeo(fn,radius,center,run=True,debug=False):
	
	"""Upates parameters in default *ball.geo* file.
	
	.. note:: Debug will also activate full debugging output of gmsh. See also
	   http://gmsh.info/doc/texinfo/gmsh.html#Command_002dline-options .
	
	.. note:: For function to work, parameters in geo file need to be defined as follows:
	   
		* radius -> radius
		* center -> center_x , center_y
	
	Args:
		fn (str): Filepath.
		radius (float): New ball radius.
		center (list): New ball center.
	
	Keyword Args:
		run (bool): Run gmsh on updated file.
		debug (bool): Print debugging messages.
		
	Returns:
		str: Path to mesh file.
		
	""" 
	
	
	v=5*int(debug)
		
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"radius",radius)
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"center_x",center[0])
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"center_y",center[1])

	if run:
		gmshBin=getGmshBin()
		os.system(gmshBin + "  -v " + str(v) +" -3 " + fn)
	
	fn_msh=fn.replace(".geo",".msh")
	
	return fn_msh




def updateDomeGeo(fn,radius,slice_height,center,run=False,debug=False):
	
	"""Upates parameters in default *dome.geo* file.
	
	.. note:: Debug will also activate full debugging output of gmsh. See also
	   http://gmsh.info/doc/texinfo/gmsh.html#Command_002dline-options .
	
	.. note:: The way that *dome.geo* is written, gmsh will automatically compute 
	   the dome geometry from slice_height and radius.
	
	.. note:: For function to work, parameters in geo file need to be defined as follows:
	   
		* radius -> radius
		* slice_height -> slice_height
		* center -> center_x , center_y
	
	
	Args:
		fn (str): Filepath.
		radius (float): New dome imaging radius.
		slice_height (float): Height of imaging slice.
		center (list): New dome center.
	
	Keyword Args:
		run (bool): Run gmsh on updated file.
		debug (bool): Print debugging messages.
		
	Returns:
		str: Path to mesh file.
		
	""" 
	
	
	v=5*int(debug)
	
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"radius",radius)
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"slice_height",slice_height)
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"center_x",center[0])
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"center_y",center[1])
	
	if run:
		gmshBin=getGmshBin()
		os.system(gmshBin + "  -v " + str(v) +" -3 " + fn)
	
	fn_msh=fn.replace(".geo",".msh")
	
	return fn_msh

def updateVolSizeGeo(fn,volSize_px,run=False,debug=False):
	
	"""Upates parameter that defines mesh element volume in *.geo* file.
	
	.. note:: Debug will also activate full debugging output of gmsh. See also
	   http://gmsh.info/doc/texinfo/gmsh.html#Command_002dline-options .
	
	.. note:: For function to work, parameters in geo file need to be defined as follows:
	   
		* volSize_px -> volSize_px
	
	Args:
		fn (str): Filepath.
		volSize_px (float): New mesh element size.
	
	Keyword Args:
		run (bool): Run gmsh on updated file.
		debug (bool): Print debugging messages.
		
	Returns:
		str: Path to mesh file.
		
	""" 
	
	v=5*int(debug)
	pyfrp_gmsh_IO_module.updateParmGeoFile(fn,"volSize_px",volSize_px)
	
	if run:
		gmshBin=getGmshBin()
		os.system(gmshBin + "  -v " + str(v) +" -3 " + fn)
	
	fn_msh=fn.replace(".geo",".msh")
	
	return fn_msh
	


def refineMsh(fn,debug=False):
	
	"""Refines mesh by splitting elements.
	
	.. note:: Debug will also activate full debugging output of gmsh. See also
	   http://gmsh.info/doc/texinfo/gmsh.html#Command_002dline-options .
	
	Args:
		fn (str): Filepath.
		
	Keyword Args:
		debug (bool): Print debugging messages.
		
	Returns:
		str: Path to mesh file.
		
	""" 
	
	v=5*int(debug)
	
	gmshBin=getGmshBin()
	os.system(gmshBin+" -v "+ str(v) + " -refine " + fn)
	return fn

def runGmsh(fn,fnOut=None,debug=False,redirect=False,fnStout=None,fnSterr=None,volSizeMax=None,dim=3):
	
	"""Runs Gmsh generating mesh from .geo file.
	
	.. note:: Debug will also activate full debugging output of gmsh. See also
	   http://gmsh.info/doc/texinfo/gmsh.html#Command_002dline-options .
	
	.. note:: If ``redirect=True``, but ``fnStout`` or  ``fnSterr`` is not specified,
	   will dump stout/sterr into ``meshfiles/gmshLogs/``.
	
	.. note:: Gmsh is run with the following settings (if all flags are activated):
	   ``gmsh -v -3 -optimize -algo3d -clmax volSizeMax -o fnOut fn``
	   This requires that Gmsh was compiled with TetGen algorithm. PyFRAP can be installed with
	   Gmsh + TetGen included by choosing the ``--gmsh`` flag. 
	   See also http://pyfrap.readthedocs.org/en/latest/setup.html#pyfrap-setup-py-api .
	
	Args:
		fn (str): Filepath.
		
	Keyword Args:
		fnOut (str): Output filepath.
		debug (bool): Print debugging messages.
		redirect (bool): Redirect gmsh stout/sterr into seperate files.
		fnStout (str): File for gmsh stout.
		fnSterr (str): File for gmsh sterr.
		volSizeMax (float): Maximum allowed mesh element size.
		dim (int): Dimension of mesh.
		
	Returns:
		str: Path to mesh file.
		
	""" 
	
	#Define where to put log files if necessary
	if fnStout==None:
		fnStout=pyfrp_misc_module.getMeshfilesDir()+'gmshLogs/gmsh.stout'
		
	if fnSterr==None:
		fnSterr=pyfrp_misc_module.getMeshfilesDir()+'gmshLogs/gmsh.sterr'
		
	v=5*int(debug)
	
	#POpen needs to have paths with `/` as seperator, so we need to change in the case of 
	#Windows the path
	if platform.system() in ["Windows"]:
		fn=pyfrp_misc_module.win2linPath(fn)
		if fnOut!=None:
			fnOut=pyfrp_misc_module.win2linPath(fnOut)
	
	#Define which command to execute
	gmshBin=getGmshBin()
	cmd = gmshBin + " -v " + str(v) +" -"+str(dim)+ " -optimize -algo del3d"
	if volSizeMax!=None:
		cmd = cmd + " -clmax " + str(volSizeMax)
	if fnOut!=None:
		cmd = cmd + " -o " + fnOut + " "
	else:
		fnOut=fn.replace('.geo','.msh')
	cmd = cmd+ " " + fn
	
	#Print out what will be done
	if debug:
		print "Will execute:"
		print cmd
	
	#Split command in list for subprocess
		
		
	args = shlex.split(cmd)
	
	#redirect stdout and stderr if selected
	if redirect:
		stoutFile = open(fnStout,'wb')
		sterrFile = open(fnSterr,'wb')
	else:	
		stoutFile = None
		sterrFile = None
		
	#Call gmsh via subprocess and wait till its done
	try:
		p = subprocess.Popen(args,stdout=stoutFile,stderr=sterrFile)
		p.wait()
	except:
		printError("Gmsh is not running properly, something is wrong.")
	
	#Fix path to make sure that it exists and is consistent with OS
	fnOut=pyfrp_misc_module.fixPath(fnOut)
	
	return fnOut

def getGmshBin(fnPath=None):
	
	"""Returns path to Gmsh binary defined in *path* file.	
	""" 
	
	return pyfrp_misc_module.getPath("gmshBin",fnPath=fnPath)
	
	


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""Parameter fitting module for PyFRAP toolbox. Contains functions for the following two tasks:
	
	* Fitting a PyFRAP simulation to FRAP data analyzed by PyFRAP and saving its results in a :py:class:`pyfrp.subclasses.pyfrp_fit.fit`` instance.
	* Pinning PyFRAP simulation and data between 0 and 1.
	* Likelihood profiling functions.

"""

#===========================================================================================================================================================================
#Improting necessary modules
#===========================================================================================================================================================================

#Misc
import sys

#Numpy/Scipy
import numpy as np
from fipy import *
from scipy import interpolate
import scipy.optimize as sciopt

#PyFRAP
import pyfrp_stats_module
import pyfrp_plot_module 
import pyfrp_optimization_module 

from pyfrp_term_module import *


#matplotlib
import matplotlib.pyplot as plt

#===========================================================================================================================================================================
#Module Functions
#===========================================================================================================================================================================

#---------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#Fits scaling solution to data

def FRAPFitting(fit,debug=False,ax=None):
	
	"""Main fitting function.
	
	Fits simulation result to analyzed data.
	
	Args:
		fit (pyfrp.subclasses.pyfrp_fit): Fit object containing all important information needed.
	
	Keyword Args:
		debug (bool): Display debugging output and plots.
		ax (matplotlib.axes): Axes to display plots in.
		
	Returns:
		pyfrp.subclasses.pyfrp_fit: Performed fit.
	"""
	
	
	#Counter for function calls
	global iterations
	iterations=0

	#Building x0
	x0=fit.getX0()
	
	#Building bounds
	bnds=fit.getBounds()
	
	#------------------------------------------------------------------------------------------------------------------------------------------------------------------
	#Calling optimization algorithms
	#------------------------------------------------------------------------------------------------------------------------------------------------------------------
	
	#Calling optimizers
	if fit.optMeth=='brute':
		res=optimize.brute(FRAPObjFunc, bnds,args=(fit,debug,ax,False), full_output=bool(debug),finish=optimize.fmin)
		
	elif fit.optMeth=='Constrained Nelder-Mead':
		LBs, UBs = pyfrp_optimization_module.buildBoundLists(fit)
		x0=pyfrp_optimization_module.transformX0(x0,LBs,UBs)
		res=sciopt.fmin(pyfrp_optimization_module.constrObjFunc,x0,args=(fit,debug,ax,False),ftol=fit.optTol,maxiter=fit.maxfun,disp=bool(debug),full_output=True)
	
	elif fit.optMeth=='Anneal':
		random.seed(555)
		res=sciopt.minimize(FRAPObjFunc, x0,args=(fit,debug,ax,False), method='Anneal')
	else:
		res=sciopt.minimize(FRAPObjFunc,x0,args=(fit,debug,ax,False),method=fit.optMeth,tol=fit.optTol,options={'maxiter': fit.maxfun, 'disp': bool(debug)})
	
	#------------------------------------------------------------------------------------------------------------------------------------------------------------------
	#Run for one last time to get final fit
	#------------------------------------------------------------------------------------------------------------------------------------------------------------------
	
	if fit.optMeth=='Constrained Nelder-Mead':
		LBs, UBs = pyfrp_optimization_module.buildBoundLists(fit)
		resNew=pyfrp_optimization_module.xTransform(res[0],LBs,UBs)
		fit=FRAPObjFunc(resNew,fit,debug,ax,True)
		
	elif fit.optMeth=='brute':
		fit=FRAPObjFunc(res[0],fit,debug,ax,True)
		
	else:	
		fit=FRAPObjFunc(res.x,fit,debug,ax,True)
	
	#------------------------------------------------------------------------------------------------------------------------------------------------------------------
	#Saving results in fit object
	#------------------------------------------------------------------------------------------------------------------------------------------------------------------
		
	if fit.optMeth=='brute':
		fit.assignOptParms(res[0])
			
		fit.SSD=res[1]
		fit.success=True
		
		fit.iterations=iterations
		#In bruteforce iterations = fcalls???
		fit.fcalls=iterations
		
	elif fit.optMeth=='Constrained Nelder-Mead':
		
		LBs, UBs = pyfrp_optimization_module.buildBoundLists(fit)
		resNew=pyfrp_optimization_module.xTransform(res[0],LBs,UBs)
		
		fit.assignOptParms(resNew)
		
		fit.SSD=res[1]
		fit.success=not bool(res[4])
		fit.iterations=res[2]
		fit.fcalls=res[3]
		
	else:	
		fit.assignOptParms(res.x)
			
		fit.SSD=res.fun
		fit.success=res.success
		
		fit.iterations=res.nit
		fit.fcalls=res.nfev
		
	fit=pyfrp_stats_module.computeFitRsq(fit)

	return fit


def assignInputVariables(x,fit):
	
	"""Decodes array given to objective function to suit
	fitting options.
	
	If ``fit.fitProd`` or ``fit.fitDegr`` are selected, will
	use ``x[1]`` or ``x[2]`` as input values for degradation and
	production rate.
	
	If ``fit.equOn=True``, then will use last entry of ``x0``
	for equalization factors, otherwise will return empty list
	for equFacts.
	
	Args:
		x (list): Input vector of objective function.
		fit (pyfrp.subclasses.pyfrp_fit): Fit object.
		
	Returns:
		tuple: Tuple containing:
		
			* Dnew (float): Diffusion rate.
			* prod (float): Production rate.
			* degr (float): Degredation rate.
			* equFacts (list): Equalization factors.
	"""
	
	Dnew=x[0]
	if fit.fitProd and fit.fitDegr:
		prod=x[1]
		degr=x[2]
	
	elif fit.fitProd and not fit.fitDegr:
		prod=x[1]
		degr=fit.x0[2]
			
	elif not fit.fitProd and fit.fitDegr:
		degr=x[1]
		prod=fit.x0[1]
		
	elif not fit.fitProd and not fit.fitDegr:
		degr=fit.x0[2]
		prod=fit.x0[1]
		
	if fit.equOn:
		idx=1+int(fit.fitDegr)+int(fit.fitProd)
		equFacts=x[idx:]
	else:
		equFacts=[]
	
	return Dnew,prod,degr,equFacts

def checkInput(x,iteration,fit):
	
	"""Checks if input vector ``x`` is non-negative.
	
	Args:
		x (list): Input vector of objective function.
		iteration (int): Number of iteration.
		fit (pyfrp.subclasses.pyfrp_fit): Fit object.
		
	Returns:
		bool: ``True`` if everythings positive, ``False`` else.
	"""
	
	#x=[1]+int(fit.fitProd)*[1],fit.fitDegr]+len(fit.ROIsFitted)*[1])*x
	if min(x)<0:
		if iteration==0:
			#If initial guess is out of bounds, just do nothing
			print printWarning("Check your initial guess, some value is negative.")
			return False
		else:
			print printWarning("One of the values of x is negative")
			return False
	return True
	
def downscaleKinetics(prod,degr,rate):
	
	"""Adjusts time-scale of kinetics so prod/degr 
	will have same weight as diffusion rate.
	
	Args:
		prod (float): Production rate.
		degr (float): Degredation rate.
		rate (float): Scaling rate.
		
	Returns:
		tuple: Tuple containing:
	
			* prod (float): Scaled production rate.
			* degr (float): Scaled degredation rate.
	"""
	
	return float(prod)/rate, float(degr)/rate

def scaleTime(tvec,D,Dnew):
	
	"""Scales time vector with D/Dnew.
	
	Args:
		tvec (numpy.ndarray): Time vector.
		D (float): Relative diffusion rate.
		Dnew (float): Scaling diffusion rate.
			
	Returns:
		numpy.ndarray: Scaled time vector.
	"""
	
	return D/Dnew*tvec

def interpolateSolution(tvecData,tvecScaled,yvec):

	"""Interpolates scaled simulation vector onto data time vector.
	
	Args:
		tvecData (numpy.ndarray): Data time vector.
		tvecScaled (numpy.ndarray): Scaled simulation time vector.
		yvec (numpy.ndarray): Simulation values.
			
	Returns:
		numpy.ndarray: Scaled simulation vector.
	"""

	fscal=interpolate.interp1d(tvecScaled,yvec)
	yvecScaled=fscal(tvecData)
	return yvecScaled

def getTvecCutIndex(tvec,tCut):
	
	"""Finds last index of time vector before time vector
	exceeds tCut.
	
	Args:
		tvec (numpy.ndarray): Time vector.
		tCut (float): Time to cut at.
		
	Returns:
		 int: Last index.
		
	"""
	
	return max(np.where(tvec<tCut)[0])

def cutTvec(tvec,tCutIndex):
	
	"""Cuts time vector as index.
	
	Args:
		tvec (numpy.ndarray): Time vector.
		tCutIndex (int): Index to cut at.
		
	Returns:
		 numpy.ndarray: Cut time vector.
		
	"""

	return tvec[:tCutIndex]

def scaleROIs(fit,Dnew):
	
	"""Scales all simulation vectors of all ROIs defined in 
	``fit.ROIsFitted``.
	
	Args:
		fit (pyfrp.subclasses.pyfrp_fit): Fit object.
		Dnew (float): Scaling diffusion rate.
	
	Returns:
		tuple: Tuple containing:
			
			* fit (pyfrp.subclasses.pyfrp_fit): Updated fit object.
			* tvecScaled (numpy.ndarray): Scaled time vector.
			* tvecData (numpy.ndarray): Data time vector.
			* scaledSimVecs (list): List of scaled simulation vectors by ROI.
			* dataVecs (list): List of data vectors by ROI.
	
	"""
	
	scaledSimVecs=[]
	dataVecs=[]
		
	#Rescaling the time series by D/Dnew
	tvecScaled=scaleTime(fit.embryo.simulation.tvecSim,fit.embryo.simulation.D,Dnew)
	
	tvecData=fit.embryo.tvecData
	
	#If cutOff option is given, need to find time point and corresponding steps of cutOff
	if fit.fitCutOffT:
		fit.cutOffStepData=getTvecCutIndex(fit.embryo.tvecData,fit.cutOffT)
		fit.cutOffStepSim=getTvecCutIndex(fit.embryo.simulation.tvecSim,fit.cutOffT)
		tvecScaled=scaleTime(fit.embryo.simulation.tvecSim[:fit.cutOffStepSim],fit.embryo.simulation.D,Dnew)
		tvecData=np.array(fit.embryo.tvecData[:fit.cutOffStepData])
		
	#Interpolating new solution		
	for r in fit.ROIsFitted:
		if fit.fitPinned:
			if fit.fitCutOffT:
				scaledSimVecs.append(np.array(interpolateSolution(fit.embryo.tvecData,tvecScaled,r.simVecPinned[:fit.cutOffStepSim])))
				dataVecs.append(np.array(r.dataVecPinned[:fit.cutOffStepData]))
			else:
				scaledSimVecs.append(np.array(interpolateSolution(fit.embryo.tvecData,tvecScaled,r.simVecPinned)))
				dataVecs.append(np.array(r.dataVecPinned))
		else:
			if fit.fitCutOffT:
				scaledSimVecs.append(interpolateSolution(np.array(fit.embryo.tvecData,tvecScaled,r.simVec[:fit.cutOffStepSim])))
				dataVecs.append(np.array(r.dataVec[:fit.cutOffStepData]))
			else:
				scaledSimVecs.append(np.array(interpolateSolution(fit.embryo.tvecData,tvecScaled,r.simVec)))
				dataVecs.append(np.array(r.dataVec))
	
	return fit,tvecScaled,tvecData,scaledSimVecs,dataVecs

def addKineticsToSolution(scaledSimVecs,tvec,prod,degr):
	
	"""Adds reaction kinetics to simulation solution.
	
	Args:
		scaledSimVecs (list): List of scaled simulation vectors by ROI.
		tvec (numpy.ndarray): Data time vector.
		prod (float): Production rate.
		degr (float): Degredation rate.
	
	Returns:
		(list): List of rescaled simulation vectors by ROI.

	"""
	
	rescaledSimVecs=[]
	
	for scaledSimVec in scaledSimVecs:
	
		#Both production and degredation
		if prod>0 and degr>0:
			rescaledSimVec=scaledSimVec*np.exp(-degr*tvec)-(prod/degr)*np.exp(-degr*tvec)+(prod/degr)
		
		#Just production
		elif prod>0 and degr<=0:
			rescaledSimVec=scaledSimVec+prod*tvec
			
		#Just degradation
		elif prod<=0 and degr>0:
			rescaledSimVec=scaledSimVec*np.exp(-degr*tvec)
		
		#Neither production nor degradation
		else:
			rescaledSimVec=scaledSimVec
		
		rescaledSimVecs.append(rescaledSimVec)
		
	return rescaledSimVecs	

def computeEquFactors(dataVec,simVec):
	
	"""Computes equalization factors per ROI.
	
	The purpose of equalization factors is to account for immobile 
	fractions, adjusting the level of the simulation vector to the data
	vector. 
	
	**Check requirements**: 
	Checks if ``0.1<=dataVec[i]/simVec[i]<=3``, if not, sets = 1.
	In practical terms, this means that the volume fraction is not allowed to increase by
	more than 3fold (>3), and that the immobile fraction cannot be more than 90% (< 0.1)
	
	Args:
		dataVec (numpy.ndarray): Data vector of ROI.
		simVec (numpy.ndarray): Scaled simulation vector of ROI.
		
	Returns:
		numpy.ndarray: Array of equalization factors.

	"""
	
	#Add little Noise to concentration profiles so we don't get singularities
	simVec=simVec+1E-10
	
	#Calculate equalization factors
	equFacts=simVec/(dataVec+1E-10)
	
	""" 
	Check requirements:
	these two if statements set the lower and upper bounds for the equalization factor.
	In practical terms, this means that the volume fraction is not allowed to increase by
	more than 3fold (>3), and that the immobile fraction cannot be more than 90% (< 0.1)
	"""
	
	equFacts[np.where((equFacts>3.) + (equFacts<0.1))[0]]=1.
	
	return equFacts

def findMinEquFacts(dataVecs,simVecs): 
	
	"""Computes list of equalization factors per ROI in ``fit.ROIsFitted`` and then
	finds the one that minimizes SSD.

	Does this by:
	
		* Computing equalization factors per ROI via :py:func:`computeEquFactors`.
		* Computing SSD for each equalization factor per ROI.
		* Selecting equalization factor per ROI that minimizes SSD.
	
	Args:
		simVecs (list): List of scaled simulation vectors by ROI.
		dataVecs (list): List of data vectors by ROI.
		equFacts (list): List of equalization factors.
	
	Returns:
		tuple: Tuple containing:
		
			* equSimVecs (list): List of equalized simulation vectors.
			* equFactsFinal (list): List of optimal equalization factors by ROI.
		
	"""
	
	equFacts=[]
	
	#Compute Equalization factors
	for i in range(len(dataVecs)):
		equFacts.append(computeEquFactors(dataVecs[i],simVecs[i]))
	
	#Compute SSD for all ROIs used for fitting for all equalization factors
	SSDs=[]
	for j in range(len(equFacts[0])):
		ssds=[]
		for i in range(len(equFacts)):	
			equSimVec=simVecs[i]/equFacts[i][j]
			ssds.append(pyfrp_stats_module.computeSSD(dataVecs[i],equSimVec))
			
		SSDs.append(sum(ssds))
	
	#Compute Final Equalization Factor (the one that minimizes SSD)
	equFactFinalIdx=SSDs.index(min(SSDs))
	
	#Append optimal equalization factors 
	equFactsFinal=[]
	for i,vec in enumerate(simVecs):
		equFactsFinal.append(equFacts[i][equFactFinalIdx])
	
	return equFactsFinal
	
def equalize(dataVecs,simVecs,equFacts,fromVecs=False):
	
	"""Equalizes all simulation vectors of all ROIs defined in 
	``fit.ROIsFitted``.
	
	If ``fromVecs=True``, will use :py:func:`findMinEquFacts` to 
	compute the equalization factors that minimize SSD from a 
	set of equalization factors given by the ratio between 
	simulation and data vector.
	
	Args:
		simVecs (list): List of scaled simulation vectors by ROI.
		dataVecs (list): List of data vectors by ROI.
		equFacts (list): List of equalization factors.
		
	Keyword Args:
		fromVecs (bool): Compute equalization factors from data/simulation ratio.
	
	Returns:
		tuple: Tuple containing:
		
			* equSimVecs (list): List of equalized simulation vectors.
			* equFacts (list): List of (optimal) equalization factors by ROI.
	"""
	
	equSimVecs=[]
	
	#Compute possible equalization factors from ratio between data and simulation
	if fromVecs:
		equFacts=findMinEquFacts(dataVecs,simVecs)
	
	#Return equalized simulation vectors
	for i,vec in enumerate(simVecs):
		equSimVecs.append(vec/equFacts[i])
		
	return equSimVecs,equFacts
	
def FRAPObjFunc(x,fit,debug,ax,returnFit):
	
	"""Objective function for fitting FRAP experiments.
	
	Does the following.
	
		* Checks input using :py:func:`pyfrp.modules.pyfrp_fit_module.checkInput` .
		* Assigns input variables using :py:func:`pyfrp.modules.pyfrp_fit_module.assignInputVariables` .
		* Adjust kinetic scales using :py:func:`pyfrp.modules.pyfrp_fit_module.downscaleKinetics` .
		* Scales simulation vector using :py:func:`pyfrp.modules.pyfrp_fit_module.scaleROIs` .
		* Adds reaction kinetics using using :py:func:`pyfrp.modules.pyfrp_fit_module.addKineticsToSolution` .
		* Equalizes simulation vector using :py:func:`pyfrp.modules.pyfrp_fit_module.equalize` .
		* Computes and returns SSD.
	     
	Args:
		x (list): Input vector, consiting of [D,(prod),(degr)].
		fit (pyfrp.subclasses.pyfrp_fit): Fit object.
		debug (bool): Display debugging output and plots.
		ax (matplotlib.axes): Axes to display plots in.
		returnFit (bool): Return fit instead of SSD.
	
	Returns:
		 float: SSD of fit. Except ``returnFit==True``, then will return fit itself. 

	"""
	
	#Counting function calls
	global iterations
	iterations=iterations+1

	#Check if any variable is negative
	if not checkInput(x,iterations,fit):
		return 2*fit.SSD
	
	#Assign Input Values
	Dnew,prod,degr,equFacts = assignInputVariables(x,fit)

	#Rescaling degr and prod
	prod,degr = downscaleKinetics(prod,degr,fit.kineticTimeScale)
	
	if debug:
		print "------------------------------------------"
		print "Dnew=",Dnew, "prod=", prod, "degr=", degr, "equFacts", equFacts
	
	#Scale simulation vectors
	try:
		fit,tvecScaled,tvecData,scaledSimVecs,dataVecs = scaleROIs(fit,Dnew)
	except ValueError:
		if debug:
			printWarning("Scaling failed with Dnew = " + str(Dnew))
		
		if returnFit:
			return fit
		else:	
			return 100000000

	#Add Kinetics
	scaledSimVecs =  addKineticsToSolution(scaledSimVecs,tvecData,prod,degr)
	
	#Equalize
	if fit.equOn:
		scaledSimVecs,equFacts=equalize(dataVecs,scaledSimVecs,equFacts)
		fit.equFacts=equFacts
		
	#Compute final SSD
	ssds=[]
	for i in range(len(dataVecs)):
		ssds.append(pyfrp_stats_module.computeSSD(dataVecs[i],scaledSimVecs[i]))
	SSD=sum(ssds)
	
	#Write final fitting vectors in ROI objects
	fit.fittedVecs=scaledSimVecs
	fit.dataVecsFitted=dataVecs
	
	#Live-Plot
	if debug:
		if iterations==1:
			if ax==None:
				global fig
				global axMon
				fig,axes = pyfrp_plot_module.makeSubplot([1,1],titles=["Live Fitting Output"],sup=fit.name)
				axMon=axes[0]
			else:
				axMon=ax
				
		else:
			axMon.cla()
		
		for i,r in enumerate(fit.ROIsFitted):
			axMon.plot(tvecData,dataVecs[i],color=r.color,linestyle='-')
			axMon.plot(tvecData,scaledSimVecs[i],color=r.color,linestyle='--')
			
		plt.draw()
		plt.pause(0.00001)
	

	
	fit.SSD=SSD
	fit.tvecFit=tvecData
	
	if returnFit:
		return fit
	else:
		return SSD

def computePinVals(vec,useMin=False,useMax=False,bkgdVal=None,debug=False):
	
	"""Computes pinning values of vector.
	
	Args:
		vec (numpy.ndarray): Vector to be pinned.
		
	Keyword Args:
		useMin (bool): Use minimum value for background computation.
		useMax (bool): Use maximum value for norm value computation.
		bkgdVal (float): Use this background value instead of newly computing it.
		debug (bool): Print debugging messages.
	
	Returns:
		tuple: Tuple containing:
			
			* bkgdVal (float): Background value.
			* normVal (float): Norming value.

	"""
	
	if bkgdVal==None:
		bkgdVal = computeBkgd(vec,useMin=useMin,debug=debug)
	vec2=np.asarray(vec)-bkgdVal
	normVal = computeNorm(vec2,useMax=useMax,debug=debug)
	
	return bkgdVal,normVal

def computeBkgd(vec,useMin=False,debug=False):
	
	"""Computes background value of vector.
	
	Args:
		vec (numpy.ndarray): Vector to be pinned.
		
	Keyword Args:
		useMin (bool): Use minimum value for background computation.
		debug (bool): Print debugging messages.
	
	Returns:
		float: Background value. 
		
	
	"""
	
	minVal=min(vec)
	if debug and not useMin:
		if minVal<vec[0]:
			printWarning("First value of vector is not the smallest. You may want to use useMin=True to get an optimal background value.")
	if useMin:
		return minVal
	else:
		return vec[0]

def computeNorm(vec,useMax=False,debug=False):
	
	"""Computes norming value of vector.
	
	Args:
		vec (numpy.ndarray): Vector to be pinned.
		
	Keyword Args:
		useMax (bool): Use maximum value for norm value computation.
		debug (bool): Print debugging messages.
	
	Returns:
		float: Norming value. 
		
	"""
	
	maxVal=max(vec)
	if debug and not useMax:
		if maxVal<vec[-1]:
			printWarning("Last value of vector is not the maximum. You may want to use useMax=True to get an optimal background value.")
	if useMax:
		return maxVal
	else:
		return vec[-1]
	
def pinConc(vec,bkgdVal,normVal,axes=None,debug=False,tvec=None,color='b'):
	
	"""Substract background and normalize: Pin concentrations between 0 and 1.
	
	Args:
		vec (numpy.ndarray): Vector to be pinned.
		bkgdVal (float): Background value.
		normVal (float): Norming value.
		
	Keyword Args:
		axes (list): List of matplotlib.axes for debugging plots.
		tvec (numpy.ndarray): Time vector (only necessay when plotting).
		debug (bool): Print debugging messages.
		color (str): Color of plots.
	
	Returns:
		numpy.ndarray: Pinned vector. 
		
	"""
	
	#Copying unpinned timeseries
	vecPinned=list(vec)
		
	#Substracting background from all timeseries
	vecPinned[:]=[x-bkgdVal for x in vecPinned]
	
	if debug:
		
		if axes==None:
			fig,axes = pyfrp_plot_module.makeSubplot([1,2],titles=['After substraction','After norming'],sup="pinConc Debug Plot")
			
		if tvec==None:
			tvec=np.arange(np.shape(vecPinned))
		
		axes[0].plot(tvec,vecPinned,color=color,linestyle='-',label="pinned vector") 
		axes[0].plot(tvec,vec,color=color,linestyle='--',label="data vector") 
		
		#axes[0].set_ylim([0,1.5])
		
		plt.draw()
		
	#Dividing background from all timeseries
	vecPinnedOld=list(vecPinned)
	vecPinned[:]=[x/normVal for x in vecPinned]
	
	if debug:
		
		axes[1].plot(tvec,vecPinnedOld,color=color,linestyle='-',label="pinned vector") 
		axes[1].plot(tvec,vecPinned,color=color,linestyle='-.',label="normed vector") 
		
		plt.legend(bbox_to_anchor=(1.05, 1), loc=2, borderaxespad=0.)
		
		#pyfrp_plot_module.redraw(axes[0])
		plt.draw()
		
	return vecPinned

def computeFitLikelihoodProfiles(fit,epsPerc=0.1,steps=100,debug=False):
	
	"""Computes likelihood profile of all parameters fitted in fit.
	
	.. warning:: Since we don't yet fit the loglikelihood function, we only compute the 
	   SSD. Even though the SSD is proportional to the loglikelihood, it should be used
	   carefully.
		   
	See also :py:func:`pyfrp.modules.pyfrp_fit_module.computeLikehoodProfile`.
	
	Args:
		fit (pyfrp.subclasses.pyfrp_fit.fit): Fit object.
		
	Keyword Args:
		epsPerc (float): Percentage of variation.
		steps (int): Number of values around optimal parameter value.
		debug (bool): Show debugging messages
	
	Returns:
		tuple: Tuple containing:
		
			* names (list): List of parameter names.
			* xvaryVec (List): List of parameter variation arrays.
			* SSDsVec (list): List of corresponding SSDs. 
	
	"""
	
	x=fit.resultsToVec()
	names=fit.getFittedParameterNames()
	
	xvaryVec=[]
	SSDsVec=[]
	
	for i in range(len(x)):
		xvary,SSDs=computeLikelihoodProfile(x,fit,i,epsPerc=epsPerc,steps=steps,debug=debug)
		xvaryVec.append(xvary)
		SSDsVec.append(SSDs)
		
	return names,xvaryVec,SSDsVec
	
def computeLikelihoodProfile(xOpt,fit,idx,steps=100,epsPerc=0.1,debug=False):
	
	"""Computes likelihood profile of parameter with index idx of fit.
	
	.. warning:: Since we don't yet fit the loglikelihood function, we only compute the 
	   SSD. Even though the SSD is proportional to the loglikelihood, it should be used
	   carefully.
	
	Args:
		xOpt (list): Vector with optimal parameters.
		fit (pyfrp.subclasses.pyfrp_fit.fit): Fit object.
		idx (int): Index of parameter in xOpt of which profile is calculated.
		
	Keyword Args:
		epsPerc (float): Percentage of variation.
		steps (int): Number of values around optimal parameter value.
		debug (bool): Show debugging messages
		
	Returns:
		tuple: Tuple containing:
		
			* xvary (numpy.ndarray): Array with varied parameter.
			* SSDs (list): Corresponding SSDs. 
	
	"""
	
	xvary=np.linspace((1-epsPerc)*xOpt[idx],(1+epsPerc)*xOpt[idx],steps)
	
	SSDs=[]
	
	for xv in xvary:
		
		x=list(xOpt)
		x[idx]=xv
		
		SSDs.append(FRAPObjFunc(x,fit,debug,None,False))
	
	return xvary, SSDs
	
def plotFitLikehoodProfiles(fit,epsPerc=0.1,steps=100,debug=False,axes=None):
	
	"""Computes and plots likelihood profile of all parameters fitted in fit.
	
	.. warning:: Since we don't yet fit the loglikelihood function, we only plot the 
	   SSD. Even though the SSD is proportional to the loglikelihood, it should be used
	   carefully.
		   
	See also :py:func:`pyfrp.modules.pyfrp_fit_module.computeFitLikehoodProfiles`.
	
	Args:
		xOpt (list): Vector with optimal parameters.
		fit (pyfrp.subclasses.pyfrp_fit.fit): Fit object.
		idx (int): Index of parameter in xOpt of which profile is calculated.
		
	Keyword Args:
		epsPerc (float): Percentage of variation.
		steps (int): Number of values around optimal parameter value.
		debug (bool): Show debugging messages
		
	Returns:
		list: List of matplotlib.axes objects used for plotting.
	
	"""
	
	names,xvaryVec,SSDs=computeFitLikelihoodProfiles(fit,epsPerc=0.1,steps=100,debug=False)
	xOpt=fit.resultsToVec()
	
	if axes==None:
		fig,axes=pyfrp_plot_module.makeSubplot([int(np.ceil(len(names)/2.)),2],sup="Profile Likehoods",show=True)

	for i in range(len(names)):
		pyfrp_plot_module.plotTS(xvaryVec[i],SSDs[i],color='k',ax=axes[i],title=names[i])
	
	
	return axes
		



	


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#===========================================================================================================================================================================
#Module Description
#===========================================================================================================================================================================

"""Z-stack module for PyFRAP toolbox. Used for reading z-stack datasets and converting them into meshes
that then can be used for a FRAP simulation.

.. warning:: This module is still experimental and unfinished and should not be used. 
   Will be added as stable in further versions.

"""

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#Numpy/Scipy
import numpy as np

#Image processing
import skimage.measure as skimsr
import cv2

#matplotlib
import matplotlib.pyplot as plt

#Misc
import sys

#PyFRAP
import pyfrp_integration_module as pyfrp_integr

#===========================================================================================================================================================================
#Module Functions
#===========================================================================================================================================================================

#---------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#Deletes small patches via opening, then closes holes via closing and then returns contours and areas of contours

def getContours(img,kernel=(10,10)):

	#Define kernel
	kernel = np.ones(kernel, np.uint8)
	
	#Open to erode small patches
	thresh = cv2.morphologyEx(img, cv2.MORPH_OPEN, kernel)
	
	#Close little holes
	thresh = cv2.morphologyEx(thresh, cv2.MORPH_CLOSE,kernel, iterations=4)
	
	#Find contours
	#contours=skimsr.find_contours(thresh,0)

	thresh=thresh.astype('uint8')
	contours, hierarchy = cv2.findContours(thresh, cv2.RETR_EXTERNAL,cv2.CHAIN_APPROX_NONE)
	
	areas=[]
	for c in contours:
		areas.append(cv2.contourArea(c))
	
	return contours,thresh,areas

def fillEndStacks(n,debug=False):

	inpoints_all=[]

	if debug==1:
		fig=plt.figure()
		fig.show()
		j=1

	for i in range(len(files)):
		
		if i==0 or i==len(files)-1:
			inpoints=[]
			if fill_mode=="random":
			
				npts=fill_samples
				
				while len(inpoints)<npts:
				
					rx=random.random() 	
					ry=random.random() 	
					
					x=mpolys[i][:,1].min()+rx*(mpolys[i][:,1].max()-mpolys[i][:,1].min())
					y=mpolys[i][:,0].min()+ry*(mpolys[i][:,0].max()-mpolys[i][:,0].min())
					
					poly=zip(list(mpolys[i][:,1]),list(mpolys[i][:,0]))
					
					if point_inside_polygon(x,y,poly):
						
						inpoints.append([x,y])
			
			elif fill_mode=="regular":
				
				dx=abs(mpolys[i][:,1].min()-mpolys[i][:,1].max())
				dy=abs(mpolys[i][:,0].min()-mpolys[i][:,0].max())
				
				d=sqrt((dx*dy)/fill_samples)
					
				xvec=arange(mpolys[i][:,1].min(),mpolys[i][:,1].max(),d)
				yvec=arange(mpolys[i][:,0].min(),mpolys[i][:,0].max(),d)
				
				poly=zip(list(mpolys[i][:,1]),list(mpolys[i][:,0]))
					
				for x in xvec:
					for y in yvec:
						if point_inside_polygon(x,y,poly):
							
							inpoints.append([x,y])
				
			inpoints_all.append(inpoints)
			
			if debug==1:
			
				ax=fig.add_subplot(2,1,j)
				ax.plot(mpolys[i][:, 1], mpolys[i][:, 0],'r-', linewidth=2)
				
				for pt in inpoints:
					ax.plot(pt[0],pt[1],'g*')
				j=j+1
		
	if debug==1:
		plt.draw()
		raw_input("Done filling first and last zstack, press ENTER to continue")


#---------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#Tries to interpolate gaps between contours (experimental/still not working)

def interpolateContourGaps(contours):

		
		#Take first contour to start with
		contour=contours[0]
		
		#Grab first and last point of contour
		pStart=contour[0,:]
		pEnd=contour[-1,:]
		
		#Build List of contours that are left
		contoursLeft=list(contours)
		contoursLeft.pop(0)
		
		#Remember contour
		contourLast=contour
		
		#Remember idx last
		idxLast=1
		
		#Create result array
		ptsFinal=contours[0].copy()
		
		#Create figure and plot contour and image
		fig=plt.figure()
		fig.show()
		ax=fig.add_subplot(111)
		ax.imshow(img,cmap='Greys')
		ax.plot(contour[:,0],contour[:,1],'r')
		plt.draw()
			
		while len(contoursLeft)>0:
			
			#Bookkeeping list
			distPEnd=[]
			
			#Now loop through contoursLeft and find the one that is closest
			for contour in contoursLeft:
				
				#Get Start/Endpoint
				pStartNew=contour[0,:]
				pEndNew=contour[-1,:]
				
				#Compute distance to both end and start point
				distPEnd.append(np.linalg.norm(pEnd-pEndNew))
				distPEnd.append(np.linalg.norm(pEnd-pStartNew))
				
				#Get the closest of the two
				idx=distPEnd.index(min(distPEnd))
				
				#Choose points for interpolation
				if idx==0:
					ptsNew=contour[-3:,:]
				elif idx==1:
					ptsNew=contour[:3,:]
					
				if idxLast==0:
					ptsOld=contourLast[:3,:]
				elif idxLast==1:
					ptsOld=contourLast[-3:,:]
				
				#Concatenate interpolation points
				ptsInterp=np.concatenate((ptsOld,ptsNew),axis=0)
				
				
				ax.plot(contour[:,0],contour[:,1],'r')
				ax.plot(ptsInterp[:,0],ptsInterp[:,1],'b*')
				plt.draw()
				
				
				
				#Generate Interpolation function
				tck,u=interp.splprep(ptsInterp.T,s=0.0)
				
				#Get interpolation range
				minx=min(ptsInterp[:,0])
				miny=min(ptsInterp[:,1])
				maxx=max(ptsInterp[:,0])
				maxy=max(ptsInterp[:,1])
				
				
				
				#Evaluate over ppoints
				x_i,y_i= interp.splev(np.linspace(minx,maxx,100),tck)
				
			
				
				ptsResult=np.zeros((len(x_i),2))
				ptsResult[:,0]=x_i
				ptsResult[:,1]=y_i
				
				
				
				#Append result
				ptsFinal=np.concatenate((ptsFinal,ptsResult),axis=0)
				ptsFinal=np.concatenate((ptsFinal,contour),axis=0)
				
				#Remeber idxLast
				idxLast=idx
			
				contourLast=contour
				
				
				#ax.plot(ptsResult[:,0],ptsResult[:,1],'g')
			
				plt.draw()
				
#-------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#Checks if a point is within a polygon via crossing number algorithm

def point_inside_polygon(x,y,poly):
	#Taken from http://www.ariel.com.au/a/python-point-int-poly.html
	n = len(poly)
	inside =False

	p1x,p1y = poly[0]
	for i in range(n+1):
		p2x,p2y = poly[i % n]
		if y > min(p1y,p2y):
			if y <= max(p1y,p2y):
				if x <= max(p1x,p2x):
					if p1y != p2y:
						xinters = (y-p1y)*(p2x-p1x)/(p2y-p1y)+p1x
					if p1x == p2x or x <= xinters:
						inside = not inside
		p1x,p1y = p2x,p2y

	return inside


#=====================================================================================================================================
#Copyright
#=====================================================================================================================================

#Copyright (C) 2014 Alexander Blaessle, Patrick Mueller and the Friedrich Miescher Laboratory of the Max Planck Society
#This software is distributed under the terms of the GNU General Public License.

#This file is part of PyFRAP.

#PyFRAP is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#! /usr/bin/python

#=====================================================================================================================================
#Description
#=====================================================================================================================================

#configuration is a simple class to save PyFRAP GUI configurations.

#===========================================================================================================================================================================
#Importing necessary modules
#===========================================================================================================================================================================

#PyFRAP modules
from pyfrp.modules import pyfrp_IO_module
from pyfrp.modules.pyfrp_term_module import *
from pyfrp.modules import pyfrp_misc_module

#OS
import os
import sys
import shutil

#---------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#Config object

class configuration:
	
	#Creates new molecule object
	def __init__(self):
		
		#Recently open files
		self.recentFiles=[]
		
		#Last view
		self.plotHidden=False
		self.termHidden=False
		self.propHidden=False
		self.backup2File=False
		self.backup2Memory=True
		self.pathFile=pyfrp_misc_module.getConfDir()+"paths"
		
		#Console History
		self.consoleHistory=[]
		
	def save(self,fn=None):
		
		if fn==None:
			fn=pyfrp_misc_module.getConfDir()+"lastConfiguration"+".conf"
		
		pyfrp_IO_module.saveToPickle(self,fn=fn)	
		
	def updateVersion(self):
		
		tempConf=configuration()
		pyfrp_misc_module.updateObj(tempConf,self)
		
		return self
	
	def setPlotHidden(self,h):
		self.plotHidden=h
		return self.plotHidden
	
	def setTermHidden(self,h):
		self.termHidden=h
		return self.termHidden
	
	def setPropHidden(self,h):
		self.propHidden=h
		return self.propHidden
	
	def getPlotHidden(self,h):
		return self.plotHidden
	
	def getTermHidden(self,h):
		return self.termHidden
	
	def getPropHidden(self,h):
		return self.propHidden
	
	def setBackup2File(self,h):
		self.backup2File=h
		return self.backup2File
	
	def getBackup2File(self,h):
		return self.backup2File
	
	def setBackup2Memory(self,h):
		self.backup2Memory=h
		return self.backup2Memory
	
	def getBackup2Memory(self,h):
		return self.backup2Memory
	
	def setRecentFiles(self,r):
		self.recentFiles=r
		return self.recentFiles
	
	def getPathFile(self):
		return self.pathFile
	
	def setPathFile(self,fn):
		self.pathFile=fn
		self.copyPathFileToDefaultLocation()
		return self.pathFile
	
	def copyPathFileToDefaultLocation(self):
		
		#Backup old paths file
		defaultFile=pyfrp_misc_module.getConfDir()+"paths"
		
		shutil.copy(defaultFile,defaultFile+".backup")
		try:
			shutil.copy(self.pathFile,defaultFile)
		except:
			printWarning('Was not able to copy '+self.pathFile+' to '+ defaultFile)
			pass
			
	def backupPathFile(self):	
		
		defaultFile=pyfrp_misc_module.getConfDir()+"paths"
		
		shutil.copy(defaultFile,self.pathFile)
	
	def getRecentFiles(self,r):
		return self.recentFiles
	
	def printConfiguration(self):
		pyfrp_term_module.printAllObjAttr(self)
	
	def addRecentFile(self,fn):
	
		if fn in self.recentFiles:
			ind=self.recentFiles.index(fn)
			self.recentFiles.pop(ind)
			
		self.recentFiles.insert(0,str(fn))
		
		return self.recentFiles